/*------------------------------------------------------------------------------
 bin/djv_view/prefs_dialog.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "prefs_dialog.h"
#include <djv_ui/style.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/group.h>
#include <djv_ui/label.h>
#include <djv_ui/button.h>
#include <djv_ui/choice.h>
#include <djv_ui/menu.h>
#include <djv_ui/num_int.h>
#include <djv_ui/tab.h>
#include <djv_ui/color.h>
#include <djv_ui/prefs_dialog.h>
#include "prefs.h"
#include "cache.h"
#include "playback.h"

namespace djv_view {
namespace prefs_dialog
{

File::File()
{
  // Widgets.

  ui::group::Box * seq_group = new ui::group::Box;
  ui::button::Check * seq_widget = new ui::button::Check("Sequence");
  seq_group->label_widget(seq_widget);
  ui::label::Text * seq_text = new ui::label::Text(
    "Automatically open file sequences.");

  ui::group::Box * proxy_group = new ui::group::Box;
  ui::choice::Widget * proxy_widget = new ui::choice::Widget(label_proxy());
  proxy_widget->label(new ui::label::Widget("Proxy:"));
  proxy_group->label_widget(proxy_widget);
  ui::label::Text * proxy_text = new ui::label::Text(
    "Proxies reduce image resolution to conserve memory.");

  ui::group::Box * cache_group = new ui::group::Box;
  ui::button::Check * cache_widget = new ui::button::Check("Cache");
  cache_group->label_widget(cache_widget);
  ui::num::Int_Edit * cache_size_widget = new ui::num::Int_Edit;
  cache_size_widget->range(0, cache_size_widget->max());
  cache_size_widget->inc(100, 1000);
  ui::label::Widget * cache_label = new ui::label::Widget("MB");
  ui::menu::Popup * cache_menu = new ui::menu::Popup(cache::label_size());
  ui::label::Text * cache_text = new ui::label::Text(
    "Keep files in memory. Disable the cache to stream directly from disk.");

  // Layout.
  
  ui::layout::Vertical * layout = new ui::layout::Vertical(this);
  layout->spacing(ui::style::global()->spacing_large());

  layout->add(seq_group);
  ui::layout::Vertical * group_layout = new ui::layout::Vertical(seq_group);
  group_layout->add(seq_text);

  layout->add(proxy_group);
  group_layout = new ui::layout::Vertical(proxy_group);
  group_layout->add(proxy_text);

  layout->add(cache_group);
  group_layout = new ui::layout::Vertical(cache_group);
  ui::layout::Horizontal * h_layout = new ui::layout::Horizontal(group_layout);
  h_layout->margin(0);
  h_layout->add(cache_size_widget);
  h_layout->add(cache_menu);
  h_layout->add(cache_label);
  group_layout->add(cache_text);
  
  layout->add_stretch();
 
  // Initialize.
  
  seq_widget->set(prefs::file()->seq());
  proxy_widget->set(prefs::file()->proxy());
  cache_widget->set(prefs::file()->cache());
  cache_size_widget->set(prefs::file()->cache_size());
  
  // Callbacks.

  seq_widget->sig.set(this, seq_callback);
  proxy_widget->sig.set(this, proxy_callback);
  cache_widget->sig.set(this, cache_callback);
  cache_size_widget->sig.set(this, cache_size_callback);
  cache_menu->sig.set(this, cache_menu_callback);
}

void File::seq_callback(bool in) { prefs::file()->seq(in); }

void File::proxy_callback(int in) { prefs::file()->proxy(in); }

void File::cache_callback(bool in) { prefs::file()->cache(in); }
void File::cache_size_callback(int in) { prefs::file()->cache_size(in); }
void File::cache_menu_callback(int in)
{ cache_size_callback(cache::size_default()[in]); }

Window::Window()
{
  // Widgets.
  
  ui::group::Box * resize_group = new ui::group::Box;
  ui::button::Check * resize_widget = new ui::button::Check("Resize");
  resize_group->label_widget(resize_widget);
  ui::label::Text * resize_text = new ui::label::Text(
    "Resize window to fit image.");
  
  ui::group::Box * controls_group = new ui::group::Box;
  ui::button::Check * controls_widget = new ui::button::Check("Controls");
  controls_group->label_widget(controls_widget);
  ui::label::Text * controls_text = new ui::label::Text(
    "Show toolbar buttons and playback controls.");
  
  // Layout.
  
  ui::layout::Vertical * layout = new ui::layout::Vertical(this);
  layout->spacing(ui::style::global()->spacing_large());
  
  layout->add(resize_group);
  ui::layout::Vertical * group_layout = new ui::layout::Vertical(resize_group);
  group_layout->add(resize_text);
  
  layout->add(controls_group);
  group_layout = new ui::layout::Vertical(controls_group);
  group_layout->add(controls_text);

  layout->add_stretch();
  
  // Initialize.
  
  resize_widget->set(prefs::window()->resize());
  controls_widget->set(prefs::window()->controls());

  // Callbacks.

  resize_widget->sig.set(this, resize_callback);
  controls_widget->sig.set(this, controls_callback);
}

void Window::resize_callback(bool in) { prefs::window()->resize(in); }
void Window::controls_callback(bool in) { prefs::window()->controls(in); }

View::View()
{
  // Widgets.
  
  ui::group::Box * grid_group = new ui::group::Box;
  ui::choice::Widget * grid_widget = new ui::choice::Widget(view::label_grid());
  grid_widget->label(new ui::label::Widget("Grid:"));
  grid_group->label_widget(grid_widget);
  _grid_swatch = new ui::color::Swatch;
  ui::color::Widget * grid_color_widget = new ui::color::Widget;
  
  ui::group::Box * hud_group = new ui::group::Box;
  ui::button::Check * hud_widget = new ui::button::Check(
    "Heads Up Display (HUD)");
  hud_group->label_widget(hud_widget);
  _hud_foreground_swatch = new ui::color::Swatch;
  ui::color::Widget * hud_foreground_widget = new ui::color::Widget;
  _hud_background_swatch = new ui::color::Swatch;
  ui::color::Widget * hud_background_widget = new ui::color::Widget;

  ui::group::Box * background_group = new ui::group::Box("Background");
  _background_swatch = new ui::color::Swatch;
  ui::color::Widget * background_widget = new ui::color::Widget;
  
  // Layout.
  
  ui::layout::Vertical * layout = new ui::layout::Vertical(this);
  layout->spacing(ui::style::global()->spacing_large());
  
  layout->add(grid_group);
  ui::layout::Vertical * group_layout = new ui::layout::Vertical(grid_group);
  ui::layout::Horizontal * h_layout = new ui::layout::Horizontal(group_layout);
  h_layout->margin(0);
  h_layout->add(_grid_swatch);
  ui::layout::Vertical * color_layout = new ui::layout::Vertical(h_layout);
  h_layout->stretch(color_layout);
  color_layout->margin(0);
  color_layout->add(grid_color_widget);
  color_layout->add_stretch();
  
  layout->add(hud_group);
  group_layout = new ui::layout::Vertical(hud_group);
  h_layout = new ui::layout::Horizontal(group_layout);
  h_layout->margin(0);
  h_layout->add(_hud_foreground_swatch);
  color_layout = new ui::layout::Vertical(h_layout);
  h_layout->stretch(color_layout);
  color_layout->margin(0);
  color_layout->add(hud_foreground_widget);
  h_layout = new ui::layout::Horizontal(group_layout);
  h_layout->margin(0);
  h_layout->add(_hud_background_swatch);
  color_layout = new ui::layout::Vertical(h_layout);
  h_layout->stretch(color_layout);
  color_layout->margin(0);
  color_layout->add(hud_background_widget);
  color_layout->add_stretch();
  
  layout->add(background_group);
  group_layout = new ui::layout::Vertical(background_group);
  h_layout = new ui::layout::Horizontal(group_layout);
  h_layout->margin(0);
  h_layout->add(_background_swatch);
  color_layout = new ui::layout::Vertical(h_layout);
  h_layout->stretch(color_layout);
  color_layout->margin(0);
  color_layout->add(background_widget);
  color_layout->add_stretch();
  
  layout->add_stretch();
  
  // Initialize.
  
  grid_widget->set(prefs::view()->grid());
  _grid_swatch->set(prefs::view()->grid_color());
  grid_color_widget->set(prefs::view()->grid_color());

  hud_widget->set(prefs::view()->hud());
  _hud_foreground_swatch->set(prefs::view()->hud_foreground());
  hud_foreground_widget->set(prefs::view()->hud_foreground());
  _hud_background_swatch->set(prefs::view()->hud_background());
  hud_background_widget->set(prefs::view()->hud_background());

  _background_swatch->set(prefs::view()->background());
  background_widget->set(prefs::view()->background());

  // Callbacks.

  grid_widget->sig.set(this, grid_callback);
  grid_color_widget->sig.set(this, grid_callback);

  hud_widget->sig.set(this, hud_callback);
  hud_foreground_widget->sig.set(this, hud_foreground_callback);
  hud_background_widget->sig.set(this, hud_background_callback);

  background_widget->sig.set(this, background_callback);
}

void View::grid_callback(int in) { prefs::view()->grid(view::GRID(in)); }

void View::grid_callback(const image::Color & in)
{
  _grid_swatch->set(in);
  prefs::view()->grid_color(in);
}

void View::hud_callback(bool in) { prefs::view()->hud(in); }

void View::hud_foreground_callback(const image::Color & in)
{
  _hud_foreground_swatch->set(in);
  prefs::view()->hud_foreground(in);
}

void View::hud_background_callback(const image::Color & in)
{
  _hud_background_swatch->set(in);
  prefs::view()->hud_background(in);
}

void View::background_callback(const image::Color & in)
{
  _background_swatch->set(in);
  prefs::view()->background(in);
}

Image::Image()
{
  // Widgets.

  ui::group::Box * transform_group = new ui::group::Box("Transform");
  
  ui::button::Check * mirror_x_widget = new ui::button::Check("Horizontal");
  ui::button::Check * mirror_y_widget = new ui::button::Check("Vertical");
  ui::label::Widget * mirror_label = new ui::label::Widget("Mirror:");
  
  ui::choice::Widget * scale_widget = new ui::choice::Widget(
    ui::image::label_scale());
  scale_widget->label(new ui::label::Widget("Scale:"));
  
  ui::choice::Widget * rotate_widget = new ui::choice::Widget(
    ui::image::label_rotate());
  rotate_widget->label(new ui::label::Widget("Rotate:"));

  ui::group::Box * display_group = new ui::group::Box("Display");

  ui::button::Check * profile_widget = new ui::button::Check("Profile");
  
  _color_widget = new ui::choice::Widget;
  _color_widget->label(new ui::label::Widget("Color:"));

  ui::choice::Widget * channel_widget = new ui::choice::Widget(
    ui::image::label_channel());
  channel_widget->label(new ui::label::Widget("Channel:"));

  //ui::choice::Widget * filter_widget = new ui::choice::Widget(
  //  ui::image::label_filter());
  //filter_widget->label(new ui::label::Widget("Filter:"));
  
  // Layout.
  
  ui::layout::group(List<ui::layout::Item *>() << scale_widget <<
    rotate_widget << profile_widget << _color_widget << channel_widget);
    //filter_widget);
  
  ui::layout::Vertical * layout = new ui::layout::Vertical(this);
  layout->spacing(ui::style::global()->spacing_large());

  layout->add(transform_group);
  ui::layout::Vertical * group_layout = new ui::layout::Vertical(
    transform_group);
  ui::layout::Horizontal * h_layout = new ui::layout::Horizontal(group_layout);
  h_layout->margin(0);
  h_layout->add(mirror_label);
  h_layout->add(mirror_x_widget);
  h_layout->add(mirror_y_widget);
  group_layout->add(scale_widget);
  group_layout->add(rotate_widget);
  
  layout->add(display_group);
  group_layout = new ui::layout::Vertical(display_group);
  group_layout->add(profile_widget);
  group_layout->add(_color_widget);
  group_layout->add(channel_widget);
  //group_layout->add(filter_widget);
  
  layout->add_stretch();
 
  // Initialize.
  
  mirror_x_widget->set(prefs::image()->mirror_x());
  mirror_y_widget->set(prefs::image()->mirror_y());
  scale_widget->set(prefs::image()->scale());
  rotate_widget->set(prefs::image()->rotate());
  profile_widget->set(prefs::image()->profile());
  channel_widget->set(prefs::image()->channel());
  //filter_widget->set(prefs::image()->filter());
  
  widget_update();
  
  // Callbacks.

  mirror_x_widget->sig.set(this, mirror_x_callback);
  mirror_y_widget->sig.set(this, mirror_y_callback);
  scale_widget->sig.set(this, scale_callback);
  rotate_widget->sig.set(this, rotate_callback);
  profile_widget->sig.set(this, profile_callback);
  _color_widget->sig.set(this, color_callback);
  channel_widget->sig.set(this, channel_callback);
  //filter_widget->sig.set(this, filter_callback);
  
  prefs::image()->color_list_sig.set(this, color_list_callback);
}

void Image::mirror_x_callback(bool in) { prefs::image()->mirror_x(in); }
void Image::mirror_y_callback(bool in) { prefs::image()->mirror_y(in); }
void Image::scale_callback(int in)
{ prefs::image()->scale(ui::image::SCALE(in)); }
void Image::rotate_callback(int in)
{ prefs::image()->rotate(ui::image::ROTATE(in)); }

void Image::profile_callback(bool in) { prefs::image()->profile(in); }
void Image::color_callback(int in) { prefs::image()->color(in - 1); }
void Image::color_list_callback(bool) { widget_update(); }
void Image::channel_callback(int in)
{ prefs::image()->channel(ui::image::CHANNEL(in)); }
//void Image::filter_callback(int in)
//{ prefs::image()->filter(ui::image::FILTER(in)); }

void Image::widget_update()
{
  callbacks(false);
  
  _color_widget->list(List<Str>() << "Default" <<
    prefs::image()->color_name_list());
  _color_widget->set(prefs::image()->color() + 1);

  callbacks(true);
}

namespace
{

class Speed_Widget : public ui::widget::Widget {
public:

  Speed_Widget()
  {
    playback::Speed_Widget * widget = new playback::Speed_Widget;
    ui::label::Widget * label = new ui::label::Widget("Speed:");
    ui::label::Widget * label2 = new ui::label::Widget("FPS");
    
    ui::layout::Horizontal * layout = new ui::layout::Horizontal(this);
    layout->margin(0);
    layout->add(label);
    layout->add(widget);
    layout->add(label2);
    
    widget->set(djv::image::time::speed());
    
    widget->sig.set(this, widget_callback);
  }
  
  ui::Signal<const image::time::Speed &> sig;
  
private:

  DJV_CALLBACK(Speed_Widget, widget_callback, const image::time::Speed &)
    { sig.emit(in); }
};

}

Playback::Playback()
{
  // Widgets.

  ui::group::Box * start_group = new ui::group::Box;
  ui::button::Check * start_widget = new ui::button::Check("Start Playback");
  start_group->label_widget(start_widget);
  ui::label::Text * start_text = new ui::label::Text(
    "Start playback when opening files.");
  
  ui::group::Box * speed_group = new ui::group::Box;
  Speed_Widget * speed_widget = new Speed_Widget;
  speed_group->label_widget(speed_widget);
  ui::label::Text * speed_text = new ui::label::Text(
    "Default playback speed.");
  
  ui::group::Box * every_frame_group = new ui::group::Box;
  ui::button::Check * every_frame_widget = new ui::button::Check(
    "Every Frame");
  every_frame_group->label_widget(every_frame_widget);
  ui::label::Text * every_frame_text = new ui::label::Text(
    "Playback every frame.");
  
  // Layout.
  
  ui::layout::Vertical * layout = new ui::layout::Vertical(this);
  layout->spacing(ui::style::global()->spacing_large());
  
  layout->add(start_group);
  ui::layout::Vertical * group_layout = new ui::layout::Vertical(start_group);
  group_layout->add(start_text);

  layout->add(speed_group);
  group_layout = new ui::layout::Vertical(speed_group);
  group_layout->add(speed_text);

  layout->add(every_frame_group);
  group_layout = new ui::layout::Vertical(every_frame_group);
  group_layout->add(every_frame_text);
  
  layout->add_stretch();

  // Initialize.
  
  start_widget->set(prefs::playback()->start());
  every_frame_widget->set(prefs::playback()->every_frame());

  // Callbacks.

  start_widget->sig.set(this, start_callback);
  speed_widget->sig.set(this, speed_callback);
  every_frame_widget->sig.set(this, every_frame_callback);
}

void Playback::start_callback(bool in)
{ prefs::playback()->start(in); }
void Playback::speed_callback(const image::time::Speed & in)
{ djv::image::time::speed(in); }
void Playback::every_frame_callback(bool in)
{ prefs::playback()->every_frame(in); }

void init()
{
  DJV_DBG_OBJ("prefs_dialog::init");
  
  ui::prefs_dialog::global()->add("File", new prefs_dialog::File);
  ui::prefs_dialog::global()->add("Window", new prefs_dialog::Window);
  ui::prefs_dialog::global()->add("View", new prefs_dialog::View);
  ui::prefs_dialog::global()->add("Image", new prefs_dialog::Image);
  ui::prefs_dialog::global()->add("Image I/O", new ui::prefs_dialog::Image_Io);
  ui::prefs_dialog::global()->add("Playback", new prefs_dialog::Playback);
  ui::prefs_dialog::global()->add("Global", new ui::prefs_dialog::Global);
}

}
}

