/*------------------------------------------------------------------------------
 bin/djv_view/window.h

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_VIEW_WINDOW_H
#define DJV_VIEW_WINDOW_H

#include <djv_base/file.h>
#include <djv_ui/window.h>
#include "cache.h"
#include "view.h"
#include "playback.h"

namespace djv_view {
namespace window
{

//------------------------------------------------------------------------------
// File Group

class File_Group : public ui::Callback_Base
{
public:

  File_Group();

  void menu(ui::menu::Widget *, int layer, const List<Str> & layer_list,
    int proxy, bool cache);
  void toolbar(ui::layout::Horizontal *);

  ui::Signal<const File &> open_sig;
  ui::Signal<bool> reload_sig, close_sig;
  ui::Signal<int> layer_sig;
  ui::Signal<bool> layer_prev_sig, layer_next_sig;
  ui::Signal<int> proxy_sig;
  ui::Signal<bool> cache_sig;

private:

  DJV_CALLBACK(File_Group, open_callback, const File &);
  DJV_CALLBACK(File_Group, open_callback, bool);
  DJV_CALLBACK(File_Group, recent_callback, bool);
  DJV_CALLBACK(File_Group, reload_callback, bool);
  DJV_CALLBACK(File_Group, close_callback, bool);

  DJV_FL_WIDGET_CALLBACK(File_Group, _open_callback);
  DJV_FL_WIDGET_CALLBACK(File_Group, _recent_callback);
  DJV_FL_WIDGET_CALLBACK(File_Group, _reload_callback);
  DJV_FL_WIDGET_CALLBACK(File_Group, _close_callback);
  DJV_FL_WIDGET_CALLBACK(File_Group, _layer_callback);
  DJV_FL_WIDGET_CALLBACK(File_Group, _layer_prev_callback);
  DJV_FL_WIDGET_CALLBACK(File_Group, _layer_next_callback);
  DJV_FL_WIDGET_CALLBACK(File_Group, _proxy_callback);
  DJV_FL_WIDGET_CALLBACK(File_Group, _cache_callback);
  DJV_FL_WIDGET_CALLBACK(File_Group, _messages_callback);
  DJV_FL_WIDGET_CALLBACK(File_Group, _prefs_callback);
  DJV_FL_WIDGET_CALLBACK(File_Group, _about_callback);
  DJV_FL_WIDGET_CALLBACK(File_Group, _exit_callback);
  
  ui::menu::Widget * _menu;
  List<int> _menu_recent, _menu_layer, _menu_proxy;
};

//------------------------------------------------------------------------------
// Window Group

class Window_Group : public ui::Callback_Base
{
public:
  
  Window_Group();
  
  void fullscreen(bool);
  
  void menu(ui::menu::Widget *, bool fullscreen, bool controls);
  void toolbar(ui::layout::Horizontal *);
  
  ui::Signal<bool> copy_sig, close_sig, fit_sig, fullscreen_sig,
    controls_sig;

private:
  
  DJV_CALLBACK(Window_Group, new_callback, bool);
  DJV_CALLBACK(Window_Group, copy_callback, bool);
  DJV_CALLBACK(Window_Group, close_callback, bool);
  DJV_CALLBACK(Window_Group, fit_callback, bool);
  DJV_CALLBACK(Window_Group, fullscreen_callback, bool);

  DJV_FL_WIDGET_CALLBACK(Window_Group, _new_callback);
  DJV_FL_WIDGET_CALLBACK(Window_Group, _copy_callback);
  DJV_FL_WIDGET_CALLBACK(Window_Group, _close_callback);
  DJV_FL_WIDGET_CALLBACK(Window_Group, _fit_callback);
  DJV_FL_WIDGET_CALLBACK(Window_Group, _fullscreen_callback);
  DJV_FL_WIDGET_CALLBACK(Window_Group, _controls_callback);
  
  ui::button::Tool * _fullscreen_widget;
  ui::menu::Widget * _menu;
};

//------------------------------------------------------------------------------
// View Group

class View_Group : public ui::Callback_Base
{
public:
  
  View_Group();
  
  void zoom(double);
  
  void menu(ui::menu::Widget *, enum view::GRID, bool hud);
  void toolbar(ui::layout::Horizontal *);
  
  ui::Signal<const Vec2i &> move_sig;
  ui::Signal<bool> center_sig;
  ui::Signal<double> zoom_sig;
  ui::Signal<bool> zoom_reset_sig;
  ui::Signal<bool> reset_sig;
  ui::Signal<bool> fit_sig;
  ui::Signal<view::GRID> grid_sig;
  ui::Signal<bool> hud_sig;
  
private:

  DJV_CALLBACK(View_Group, zoom_in_callback, bool);
  DJV_CALLBACK(View_Group, zoom_out_callback, bool);
  DJV_CALLBACK(View_Group, zoom_inc_callback, bool);
  DJV_CALLBACK(View_Group, zoom_reset_callback, bool);
  DJV_CALLBACK(View_Group, zoom_dec_callback, bool);
  DJV_CALLBACK(View_Group, fit_callback, bool);

  DJV_FL_WIDGET_CALLBACK(View_Group, _left_callback);
  DJV_FL_WIDGET_CALLBACK(View_Group, _right_callback);
  DJV_FL_WIDGET_CALLBACK(View_Group, _up_callback);
  DJV_FL_WIDGET_CALLBACK(View_Group, _down_callback);
  DJV_FL_WIDGET_CALLBACK(View_Group, _center_callback);
  DJV_FL_WIDGET_CALLBACK(View_Group, _zoom_in_callback);
  DJV_FL_WIDGET_CALLBACK(View_Group, _zoom_out_callback);
  DJV_FL_WIDGET_CALLBACK(View_Group, _zoom_reset_callback);
  DJV_FL_WIDGET_CALLBACK(View_Group, _reset_callback);
  DJV_FL_WIDGET_CALLBACK(View_Group, _fit_callback);
  DJV_FL_WIDGET_CALLBACK(View_Group, _grid_callback);
  DJV_FL_WIDGET_CALLBACK(View_Group, _hud_callback);

  ui::label::Widget * _zoom_label;
  ui::menu::Widget * _menu;
  List<int> _menu_grid;
};

//------------------------------------------------------------------------------
// Image Group

class Image_Group : public ui::Callback_Base
{
public:
  
  Image_Group();
  
  void pick(const Vec2i &, double zoom);
  
  void menu(ui::menu::Widget *, bool mirror_x, bool mirror_y, ui::image::SCALE,
    ui::image::ROTATE, bool profile, ui::image::CHANNEL, ui::image::FILTER);
  void toolbar(ui::layout::Horizontal *);

  ui::Signal<bool> mirror_x_sig, mirror_y_sig;
  ui::Signal<ui::image::SCALE> scale_sig;
  ui::Signal<ui::image::ROTATE> rotate_sig;
  ui::Signal<bool> profile_sig;
  ui::Signal<bool> color_sig;
  ui::Signal<const Color &> color_value_sig;
  ui::Signal<bool> color_reset_sig;
  ui::Signal<const Str &> color_add_sig;
  ui::Signal<ui::image::CHANNEL> channel_sig;
  ui::Signal<ui::image::FILTER> filter_sig;
  ui::Signal<bool> magnify_sig, color_sample_sig, histogram_sig, info_sig;

private:

  DJV_CALLBACK(Image_Group, color_callback, bool);
  DJV_CALLBACK(Image_Group, color_add_callback, const Str &);
  DJV_CALLBACK(Image_Group, color_del_callback, int);
  DJV_CALLBACK(Image_Group, color_del_all_callback, bool);
  DJV_CALLBACK(Image_Group, color_list_callback, bool);
  DJV_CALLBACK(Image_Group, magnify_callback, bool);
  DJV_CALLBACK(Image_Group, color_sample_callback, bool);

  DJV_FL_WIDGET_CALLBACK(Image_Group, _mirror_x_callback);
  DJV_FL_WIDGET_CALLBACK(Image_Group, _mirror_y_callback);
  DJV_FL_WIDGET_CALLBACK(Image_Group, _scale_callback);
  DJV_FL_WIDGET_CALLBACK(Image_Group, _rotate_callback);
  DJV_FL_WIDGET_CALLBACK(Image_Group, _profile_callback);
  DJV_FL_WIDGET_CALLBACK(Image_Group, _color_callback);
  DJV_FL_WIDGET_CALLBACK(Image_Group, _color_reset_callback);
  DJV_FL_WIDGET_CALLBACK(Image_Group, _color_add_callback);
  DJV_FL_WIDGET_CALLBACK(Image_Group, _color_del_callback);
  DJV_FL_WIDGET_CALLBACK(Image_Group, _color_del_all_callback);
  DJV_FL_WIDGET_CALLBACK(Image_Group, _color_list_callback);
  DJV_FL_WIDGET_CALLBACK(Image_Group, _channel_callback);
  DJV_FL_WIDGET_CALLBACK(Image_Group, _filter_callback);
  DJV_FL_WIDGET_CALLBACK(Image_Group, _magnify_callback);
  DJV_FL_WIDGET_CALLBACK(Image_Group, _color_sample_callback);
  DJV_FL_WIDGET_CALLBACK(Image_Group, _histogram_callback);
  DJV_FL_WIDGET_CALLBACK(Image_Group, _info_callback);

  ui::label::Widget * _pick_label;
  ui::menu::Widget * _menu;
  List<int> _menu_scale, _menu_rotate, _menu_profile, _menu_color,
    _menu_channel, _menu_filter;
};

//------------------------------------------------------------------------------
// Playback Group

class Playback_Group : public ui::Callback_Base
{
public:
  
  Playback_Group();
  
  void set(playback::PLAYBACK);
  void time(const image::Time &, int in, int out);
  void frame(int);
  void speed(const image::time::Speed &, bool every_frame);
  void speed_real(double, bool dropped_frames);
  void speed_default(const image::time::Speed &);
  
  void menu(ui::menu::Widget *, playback::PLAYBACK playback, bool every_frame);
  void playbar(ui::group::Frame *);

  ui::Signal<playback::PLAYBACK> playback_sig;
  ui::Signal<bool> playback_shuttle_sig;
  ui::Signal<int> playback_shuttle_value_sig;
  ui::Signal<int> frame_sig;
  ui::Signal<playback::FRAME> frame_move_sig;
  ui::Signal<bool> frame_shuttle_sig;
  ui::Signal<int> frame_shuttle_value_sig;
  ui::Signal<int> in_sig, out_sig;
  ui::Signal<bool> in_mark_sig, in_reset_sig, out_mark_sig, out_reset_sig;
  ui::Signal<const image::time::Speed &> speed_sig;
  ui::Signal<bool> every_frame_sig;

private:
  
  DJV_CALLBACK(Playback_Group, playback_callback, playback::PLAYBACK);
  DJV_CALLBACK(Playback_Group, stop_callback, bool);
  DJV_CALLBACK(Playback_Group, forward_callback, bool);
  DJV_CALLBACK(Playback_Group, reverse_callback, bool);
  DJV_CALLBACK(Playback_Group, playback_shuttle_callback, bool);
  DJV_CALLBACK(Playback_Group, playback_shuttle_value_callback, int);
  DJV_CALLBACK(Playback_Group, frame_callback, int);
  DJV_CALLBACK(Playback_Group, frame_move_callback, playback::FRAME);
  DJV_CALLBACK(Playback_Group, frame_shuttle_callback, bool);
  DJV_CALLBACK(Playback_Group, frame_shuttle_value_callback, int);
  DJV_CALLBACK(Playback_Group, start_callback, bool);
  DJV_CALLBACK(Playback_Group, end_callback, bool);
  DJV_CALLBACK(Playback_Group, prev_callback, bool);
  DJV_CALLBACK(Playback_Group, next_callback, bool);
  DJV_CALLBACK(Playback_Group, in_callback, int);
  DJV_CALLBACK(Playback_Group, in_mark_callback, bool);
  DJV_CALLBACK(Playback_Group, in_reset_callback, bool);
  DJV_CALLBACK(Playback_Group, out_callback, int);
  DJV_CALLBACK(Playback_Group, out_mark_callback, bool);
  DJV_CALLBACK(Playback_Group, out_reset_callback, bool);
  DJV_CALLBACK(Playback_Group, speed_callback, const image::time::Speed &);
  DJV_CALLBACK(Playback_Group, every_frame_callback, bool);
  
  DJV_FL_WIDGET_CALLBACK(Playback_Group, _stop_callback);
  DJV_FL_WIDGET_CALLBACK(Playback_Group, _forward_callback);
  DJV_FL_WIDGET_CALLBACK(Playback_Group, _reverse_callback);
  DJV_FL_WIDGET_CALLBACK(Playback_Group, _start_callback);
  DJV_FL_WIDGET_CALLBACK(Playback_Group, _end_callback);
  DJV_FL_WIDGET_CALLBACK(Playback_Group, _prev_callback);
  DJV_FL_WIDGET_CALLBACK(Playback_Group, _prev_x10_callback);
  DJV_FL_WIDGET_CALLBACK(Playback_Group, _prev_x100_callback);
  DJV_FL_WIDGET_CALLBACK(Playback_Group, _next_callback);
  DJV_FL_WIDGET_CALLBACK(Playback_Group, _next_x10_callback);
  DJV_FL_WIDGET_CALLBACK(Playback_Group, _next_x100_callback);
  DJV_FL_WIDGET_CALLBACK(Playback_Group, _in_mark_callback);
  DJV_FL_WIDGET_CALLBACK(Playback_Group, _in_reset_callback);
  DJV_FL_WIDGET_CALLBACK(Playback_Group, _out_mark_callback);
  DJV_FL_WIDGET_CALLBACK(Playback_Group, _out_reset_callback);
  DJV_FL_WIDGET_CALLBACK(Playback_Group, _every_frame_callback);
  
  ui::button::Tool * _stop_widget, * _forward_widget, * _reverse_widget;
  playback::Frame_Widget * _frame_widget;
  playback::Frame_Slider * _frame_slider;
  playback::Time_Widget * _start_widget, * _end_widget;
  playback::Speed_Widget * _speed_widget;
  playback::Speed_Display_Widget * _speed_real_widget;
  ui::button::Tool * _every_frame_widget;
  ui::menu::Widget * _menu;
  List<int> _menu_playback;
};

//------------------------------------------------------------------------------
// Widget

class Widget : public ui::window::Widget
{
public:

  Widget(Widget * copy = 0);
  ~Widget();

  void file_open(const File, bool init = false);

  void window_fit(bool init = false);

  void playback(playback::PLAYBACK);
  void playback_time(const image::Time &);
  void playback_frame(int);
  void playback_frame_prev(int = 1);
  void playback_frame_next(int = 1);
  void playback_in(int);
  void playback_out(int);
  
  void widget_del();

private:

  DJV_CALLBACK(Widget, file_open_callback, const File &);
  DJV_CALLBACK(Widget, file_recent_callback, const List<Str> &);
  DJV_CALLBACK(Widget, file_reload_callback, bool);
  DJV_CALLBACK(Widget, file_reload_callback, image::io::Plugin *);
  DJV_CALLBACK(Widget, file_close_callback, bool);
  DJV_CALLBACK(Widget, file_layer_callback, int);
  DJV_CALLBACK(Widget, file_layer_prev_callback, bool);
  DJV_CALLBACK(Widget, file_layer_next_callback, bool);
  DJV_CALLBACK(Widget, file_proxy_callback, int);
  DJV_CALLBACK(Widget, file_cache_callback, bool);

  DJV_CALLBACK(Widget, window_copy_callback, bool);
  DJV_CALLBACK(Widget, window_close_callback, bool);
  DJV_CALLBACK(Widget, window_close2_callback, bool);
  DJV_CALLBACK(Widget, window_fit_callback, bool);
  DJV_CALLBACK(Widget, window_fullscreen_callback, bool);
  DJV_CALLBACK(Widget, window_controls_callback, bool);

  DJV_CALLBACK(Widget, view_callback, bool);
  DJV_CALLBACK(Widget, view_move_callback, const Vec2i &);
  DJV_CALLBACK(Widget, view_center_callback, bool);
  DJV_CALLBACK(Widget, view_zoom_callback, double);
  DJV_CALLBACK(Widget, view_zoom_reset_callback, bool);
  DJV_CALLBACK(Widget, view_reset_callback, bool);
  DJV_CALLBACK(Widget, view_fit_callback, bool);
  DJV_CALLBACK(Widget, view_grid_callback, view::GRID);
  DJV_CALLBACK(Widget, view_grid_color_callback, const image::Color &);
  DJV_CALLBACK(Widget, view_hud_callback, bool);
  DJV_CALLBACK(Widget, view_hud_background_callback, const image::Color &);
  DJV_CALLBACK(Widget, view_hud_foreground_callback, const image::Color &);
  DJV_CALLBACK(Widget, view_pick_callback, const Vec2i &);
  DJV_CALLBACK(Widget, view_background_callback, const image::Color &);

  DJV_CALLBACK(Widget, image_mirror_x_callback, bool);
  DJV_CALLBACK(Widget, image_mirror_y_callback, bool);
  DJV_CALLBACK(Widget, image_scale_callback, ui::image::SCALE);
  DJV_CALLBACK(Widget, image_rotate_callback, ui::image::ROTATE);
  DJV_CALLBACK(Widget, image_profile_callback, bool);
  DJV_CALLBACK(Widget, image_color_callback, bool);
  DJV_CALLBACK(Widget, image_color_callback, const Color &);
  DJV_CALLBACK(Widget, image_color_reset_callback, bool);
  DJV_CALLBACK(Widget, image_color_list_callback, bool);
  DJV_CALLBACK(Widget, image_color_add_callback, const Str &);
  DJV_CALLBACK(Widget, image_channel_callback, ui::image::CHANNEL);
  DJV_CALLBACK(Widget, image_filter_callback, ui::image::FILTER);
  DJV_CALLBACK(Widget, image_magnify_callback, bool);
  DJV_CALLBACK(Widget, image_color_sample_callback, bool);
  DJV_CALLBACK(Widget, image_histogram_callback, bool);
  DJV_CALLBACK(Widget, image_info_callback, bool);
  
  DJV_CALLBACK(Widget, playback_callback, playback::PLAYBACK);
  DJV_CALLBACK(Widget, playback_shuttle_callback, bool);
  DJV_CALLBACK(Widget, playback_shuttle_value_callback, int);
  DJV_CALLBACK(Widget, playback_frame_callback, int);
  DJV_CALLBACK(Widget, playback_frame_move_callback, playback::FRAME);
  DJV_CALLBACK(Widget, playback_frame_shuttle_callback, bool);
  DJV_CALLBACK(Widget, playback_frame_shuttle_value_callback, int);
  DJV_CALLBACK(Widget, playback_in_callback, int);
  DJV_CALLBACK(Widget, playback_in_mark_callback, bool);
  DJV_CALLBACK(Widget, playback_in_reset_callback, bool);
  DJV_CALLBACK(Widget, playback_out_callback, int);
  DJV_CALLBACK(Widget, playback_out_mark_callback, bool);
  DJV_CALLBACK(Widget, playback_out_reset_callback, bool);
  DJV_CALLBACK(Widget, playback_speed_callback, const image::time::Speed &);
  DJV_CALLBACK(Widget, playback_every_frame_callback, bool);
  DJV_FL_CALLBACK(Widget, playback_timer_callback);

  void file_cache_update();
  
  void window_controls_update();
  void window_fullscreen_update();
  
  void view_update();
  void view_overlay_update();
  
  void image_update();
  void image_display_update();
  void image_resize_update();
  void image_dialog_update();
  void image_pick_update();
  
  void playback_update();
  void playback_time_update();
  void playback_frame_update();
  void playback_speed_update();

  void menu_update();

  File _file;
  List<image::Info> _info;
  image::Tag _tag;
  image::Time _time;
  std::auto_ptr<image::io::Load> _load;
  int _file_layer, _file_proxy;
  List<Str> _file_layer_list;
  bool _file_cache;
  std::auto_ptr<File_Group> _file_group;

  bool _window_controls;
  std::auto_ptr<Window_Group> _window_group;

  view::Widget * _view;
  view::GRID _view_grid;
  bool _view_hud;
  std::auto_ptr<View_Group> _view_group;

  const image::Image * _image_p;
  cache::Ref * _image_ref;
  bool _image_mirror_x, _image_mirror_y;
  ui::image::SCALE _image_scale;
  ui::image::ROTATE _image_rotate;
  bool _image_profile;
  Color _image_color;
  ui::Signal<const Color &> _image_color_sig;
  ui::image::CHANNEL _image_channel;
  ui::image::FILTER _image_filter;
  Vec2i _image_pick;
  std::auto_ptr<Image_Group> _image_group;

  playback::PLAYBACK _playback_mode;
  image::Time _playback_time;
  int _playback_frame, _playback_frame_tmp, _playback_in, _playback_out;
  bool _playback_every_frame;
  bool _playback_shuttle;
  double _playback_shuttle_speed;
  double _playback_speed_real;
  bool _playback_dropped_frames, _playback_dropped_frames_tmp;
  bool _playback_timer_init;
  djv::time::Timer _playback_timer, _playback_speed_timer;
  uint64_t _playback_timer_frame, _playback_speed_counter;
  std::auto_ptr<Playback_Group> _playback_group;
  
  ui::menu::Widget * _menu;
  ui::group::Frame * _toolbar, * _playbar;
};

//------------------------------------------------------------------------------
// Window Factory

class Factory
{
public:

  ~Factory();

  Widget * add(Widget * copy = 0);
  void del(Widget *);
  const List<Widget *> list() const { return _list; }

private:
  
  List<Widget *> _list;
};

//------------------------------------------------------------------------------
// Global

Factory * factory();

void factory_del();

}
}

#endif

