/*------------------------------------------------------------------------------
 bin/djv_vlut/app.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "app.h"
#include <djv_base/sys.h>
#include <djv_ui/prefs.h>
#include <djv_ui/prefs_dialog.h>

namespace djv_vlut
{

App::App(int & argc, char ** argv) :
  ui::App("djv_vlut", argc, argv),
  _input("0.vlut"),
  _window(0)
{
  DJV_DBG_OBJ("App::App");
  
  // Command line.

  if (app::EXIT_DEFAULT == image::App::exit())
    try { cmdln(); }
    catch (Error in)
    {
      error(str::Format("Command Line: %1").arg(in));
      image::App::exit(app::EXIT_ERROR);
      return;
    }
  if (image::App::exit() != app::EXIT_DEFAULT) return;
  
  // Initialize.
  
  valid(true);
  
  ui::prefs::Prefs * prefs = ui::prefs::app();
  List<Str> tmp;
  if (prefs->get("value", &tmp))
  {
    serialize(&tmp, &_color);
    _color.lut_load();
  }

  ui::prefs_dialog::global()->add("Image I/O", new ui::prefs_dialog::Image_Io);
  ui::prefs_dialog::global()->add("Global", new ui::prefs_dialog::Global);

  try
  {
    image::Info info;
    std::auto_ptr<image::io::Load> load(image::io::load(_input, &info, 0, 0,
      "vlut"));
    DJV_DBG_PRN("vlut = " << info);
    
    _image.set(image::Info(info.size, info.pixel));
    _image_tmp.set(image::Info(info.size, info.pixel));

    _save = std::auto_ptr<image::io::Save>(
      image::io::save(_input, info, image::Tag(), image::Time(), "vlut"));
  }
  catch (Error in)
  {
    error(str::Format("%1: %2").arg(in).arg(_input));
  }
  
  _window = new window::Widget;
  
  color_update();

  _window->show();
  
  // Callbacks.
  
  _window->sig.set(this, color_callback);
}

App::~App()
{
  DJV_DBG_OBJ("App::~App");
  
  if (_window) _window->widget_del();
  
  color_list_del();
  
  ui::prefs::Prefs * prefs = ui::prefs::app();
  prefs->set("value", serialize(_color));
}

void App::color_callback(const Color & in)
{
  _color = in;
  color_update();
}

void App::color_update()
{
  DJV_DBG_OBJ("App::color_update");
  DJV_DBG_PRN("color = " << _color);
  
  try
  {
    image::gl::data::gradient(&_image);
    image::gl::data::Draw draw;
    draw.display = _color;
    image::gl::data::copy(_image, &_image_tmp, draw);
    
    /*const image::pixel::F32_T * p =
      reinterpret_cast<const image::pixel::F32_T *>(_image_tmp.data());
    for (int i = 0; i < _image_tmp.size().x; ++i, ++p)
      DJV_DBG_PRN(i << " = " << *p);*/

    if (_save.get()) _save->set(_image_tmp);
  }
  catch (Error in)
  {
    error(in);
  }
  
  _window->set(_color);
}

void App::cmdln() throw (Error)
{
  if (0 == cmdln_list.size()) ;
  else if (1 == cmdln_list.size()) _input = cmdln_list[0];
  else throw Error(cmdln_list[1]);
}

namespace
{
const Str _help =
"\n"
"djv_vlut\n"
"--------\n"
"\n"
"Video lookup table utility.\n"
"\n"
"Command Line\n"
"------------\n"
"\n"
"Usage:\n"
"\n"
"djv_vlut [input] [option]...\n"
"\n"
"input - Input video lookup table. Default: %1.\n"
"\n"
"%2";
}

Str App::help() const
{
  return str::Format(_help).
    arg(_input).
    arg(ui::App::help());
}

}

int main(int argc, char ** argv)
{
  int r = 1;
  try { r = djv_vlut::App(argc, argv).exec(); }
  catch (djv::Error in) { djv::error::print(in); }
  return r;
}

