/*------------------------------------------------------------------------------
 lib/djv_base/dir.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_base/dir.cxx"

#include "dir.h"
#include <FL/filename.H>
#include <stdlib.h>

namespace djv {
namespace dir
{

Dir::Dir() : _valid(false)
{}

Dir::Dir(const Str & in) : _valid(false)
{
  set(in);
}

void Dir::set(const Str & in)
{
  DJV_DBG_OBJ2(_DBG, "Dir::set");
  DJV_DBG_PRN("in = " << in);
  
  _path = file::path_fix(in);
  DJV_DBG_PRN("path = " << _path);
  
  update();
}

List<File> Dir::list() const
{
  DJV_DBG_OBJ2(_DBG, "Dir::list");
  DJV_DBG_PRN("path = " << _path);

  List<File> out;

  dirent ** dirent_list = 0;
  const int size = fl_filename_list(_path.c_str(), &dirent_list, fl_alphasort);
  DJV_DBG_PRN("size = " << size);
  if (-1 == size) return out;
  out.resize(size);

  int j = 0;
  for (int i = 0; i < size; ++i)
  {
    const char * name = dirent_list[i]->d_name;
    const char * p = name;
    for (; *p; ++p) ;
    if (p - name)
      if ('/' == *(p - 1) || '\\' == *(p - 1)) --p;
    if (1 == (p - name))
    {
      if ('.' == name[0]) name = 0;
    }
    else if (2 == (p - name))
    {
      if ('.' == name[0] && '.' == name[1]) name = 0;
    }
    
    if (name)
    {
      DJV_DBG_PRN("file = " << name);
      out[j++] = File(_path + Str(name, p - name));
    }

    free((void *)dirent_list[i]);
  }
  if (dirent_list) free((void *)dirent_list);
  if (j < size) out.resize(j);

  DJV_DBG_PRN("size = " << out.size());
  return out;
}

void Dir::cd(const Str & in)
{
  DJV_DBG_OBJ2(_DBG, "Dir::cd");
  DJV_DBG_PRN("in = " << in);

  _path = file::path_fix(_path + in);
  
  update();
}

void Dir::up()
{
  cd("..");
}

void Dir::update()
{
  dirent ** dirent_list = 0;
  const int size = fl_filename_list(get().c_str(), &dirent_list, fl_alphasort);
  _valid = size >= 0;
  for (int i = 0; i < size; ++i)
    ::free((void *)dirent_list[i]);
  if (dirent_list) ::free((void *)dirent_list);
}

}
}
