/*------------------------------------------------------------------------------
 lib/djv_base/file_fnc.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_base/file_fnc.cxx"

#include "file.h"
#include <djv_base/math.h>

namespace djv {
namespace file
{

Str seq_to_str(const Seq & in)
{
  //DJV_DBG_OBJ2(_DBG, "seq_to_str");
  //DJV_DBG_PRN("in = " << in.start << "-" << in.end << ":" << in.pad);
  
  Str out = frame_to_str(in.start, in.pad);
  if (in.start != in.end)
  {
    out += '-';
    out += frame_to_str(in.end, in.pad);
  }

  //DJV_DBG_PRN("out = " << out);
  return out;
}

Seq str_to_seq(const Str & in)
{
  //DJV_DBG_OBJ2(_DBG, "str_to_seq");
  //DJV_DBG_PRN("in = " << in);

  Seq out;
  const Str::size_type i = in.find('-');
  if (i != Str::npos)
  {
    const Str start(in, 0, i), end(in, i + 1, in.size() - (i + 1));
    //DJV_DBG_PRN("start = " << start);
    //DJV_DBG_PRN("end = " << end);
    
    int pad = 0;
    out.start = str_to_frame(start, &pad);
    out.pad = pad;
    out.end = str_to_frame(end, &pad);
    out.pad = math::max(pad, out.pad);
  }
  else
  {
    out.start = out.end = str_to_frame(in, &out.pad);
  }

  //DJV_DBG_PRN("out = " << out.start << "-" << out.end << ":" << out.pad);
  return out;
}

Str frame_to_str(int in, int pad)
{
  char format [] = "%00d";
  static const int format_max = 9;
  format[2] = char('0' + math::min(pad, format_max));
  
  char tmp [str::cstr_size];
  const int size = SNPRINTF(tmp, str::cstr_size, format, in);
  return Str(tmp, size);
}

const List<Str> & label_type()
{
  static const List<Str> data = List<Str>() << "File" << "Seq" << "Dir";
  return data;
}

Str label_size(uint64_t in)
{
  if (in > mem::terabyte)
    return str::Format("%1TB").arg(in / double(mem::terabyte), 0, -2);
  else if (in > mem::gigabyte)
    return str::Format("%1GB").arg(in / double(mem::gigabyte), 0, -2);
  else if (in > mem::megabyte)
    return str::Format("%1MB").arg(in / double(mem::megabyte), 0, -2);
  else
    return str::Format("%1KB").arg(in / double(mem::kilobyte), 0, -2);
}

const Str & label_perm(int in)
{
  static const Str data [] = { "", "r", "w", "rw", "x", "rx", "wx", "rwx" };
  return data[in];
}

const List<Str> & label_sort()
{
  static const List<Str> data = List<Str>() << "Name" << "Type" << "Size" <<
    "User" << "Permissions" << "Time";
  return data;
}

bool operator == (const File & a, const File & b) { return a.get() == b.get(); }
bool operator != (const File & a, const File & b) { return a.get() != b.get(); }

Dbg & operator << (Dbg & dbg, const Seq & in)
{ return dbg << str::Format("%1-%2:%3").arg(in.start).arg(in.end).
  arg(in.pad); }

Dbg & operator << (Dbg & dbg, const File & in)
{ return dbg << in.get(); }

}
}

