/*------------------------------------------------------------------------------
 lib/djv_base/math_.h

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

namespace djv {
namespace math
{

template<class T>
inline T abs(T in)
{
  return in < T(0) ? -in : in;
}

template<class T>
inline T mod(T in, T mod)
{
  T tmp = in;
  if (mod != T(0))
  {
    tmp = in - (int(in / mod) * mod);
    if (tmp < T(0)) tmp += mod;
  }
  return tmp;
}

inline int wrap(int in, int min, int max)
{
  return min + mod(in - min, max - min + 1);
}

inline int step(int in, int v) { return in <= v; }

inline int pulse(int in, int min, int max)
{
  return step(min, in) - step(max, in);
}

template<class T>
inline T min(T a, T b)
{
  return a < b ? a : b;
}

template<class T>
inline T max(T a, T b)
{
  return a > b ? a : b;
}

template<class T>
inline T clamp(T in, T min, T max)
{
  return in < min ? min : (in > max ? max : in);
}

template<class T, class U>
inline T lerp(U in, T min, T max)
{
  return min + T(in * (max - min));
}

inline double smooth_step(double in, double min, double max)
{
  return lerp(in * in * (3.0 - (2.0 * in)), min, max);
}

template<class T>
inline bool intersect(T in, T min, T max)
{
  return in >= min && in < max;
}

inline bool intersect(int in, int min, int max)
{
  return in >= min && in < max + 1;
}

template<class T>
inline bool intersect(T min_a, T max_a, T min_b, T max_b, T * min, T * max)
{
  *min = math::max(min_a, min_b);
  *max = math::min(max_a, max_b);
  return *max > *min;
}

inline bool intersect(int min_a, int max_a, int min_b, int max_b,
  int * min, int * max
) {
  *min = math::max(min_a, min_b);
  *max = math::min(max_a, max_b);
  return (*max + 1) > *min;
}

template<class T>
inline void bound(T min_a, T max_a, T min_b, T max_b, T * min, T * max)
{
  *min = math::min(min_a, min_b);
  *max = math::max(max_a, max_b);
}

/*inline int ceil(double in)
{
  const int tmp = int(in);
  return tmp + (in > 0 && in != tmp);
}

inline int floor(double in)
{
  const int tmp = int(in);
  return tmp - (in < 0 && ! in != tmp);
}*/

inline double fraction(double in) { return in - int(in); }

inline int to_pow2(int in)
{
  int i = 0;
  for (; in > (1 << i); ++i) ;
  return 1 << i;
}

inline double degrees_to_radians(double in) { return in * 0.017453; }
inline double radians_to_degrees(double in) { return in * 57.29578; }

}
}
