/*------------------------------------------------------------------------------
 lib/djv_base/mem_.h

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include <malloc.h>

namespace djv {
namespace mem
{

template<typename T>
inline Buffer<T>::Buffer() : _data(0), _size(0)
{}

template<typename T>
inline Buffer<T>::Buffer(const Buffer & in) : _data(0), _size(0)
{
  *this = in;
}

template<typename T>
inline Buffer<T>::Buffer(size_t in) : _data(0), _size(0)
{
  size(in);
}

template<typename T>
inline Buffer<T>::~Buffer()
{
  del();
}

template<typename T>
inline Buffer<T> & Buffer<T>::operator = (const Buffer<T> & in)
{
  if (&in != this)
  {
    size(in._size);
    copy(in._data, _data, _size);
  }
  return *this;
}

template<typename T>
inline void Buffer<T>::size(size_t in)
{
  if (in != _size)
  {
    del();
    _size = in;
    //_data = new T [_size + 1];
#if ! defined(WIN32)
    _data = reinterpret_cast<T *>(
      ::memalign(512, _size * sizeof(T) + 1));
#else
    _data = reinterpret_cast<T *>(
      ::malloc(_size * sizeof(T) + 1));
#endif
    zero();
  }
}

template<typename T>
inline void Buffer<T>::del()
{
  if (_data)
  {
    //delete [] _data;
    free(_data);
    _data = 0;
    _size = 0;
  }
}

template<typename T>
inline void Buffer<T>::zero()
{
  mem::zero(_data, _size * sizeof(T));
}

inline ENDIAN endian()
{
  static const int tmp = 1;
  static const uint8_t * const p = reinterpret_cast<const uint8_t *>(&tmp);
  return *p ? LSB : MSB;
}

inline void endian(void * in, size_t size, size_t word_size)
{
  uint8_t * p = reinterpret_cast<uint8_t *>(in);
  uint8_t tmp;
  
  switch (word_size)
  {
    case 2:
      while (size--) {
        tmp = p[0];
        p[0] = p[1];
        p[1] = tmp;
        p += 2;
      }
      break;

    case 4:
      while (size--) {
        tmp = p[0];
        p[0] = p[3];
        p[3] = tmp;
        tmp = p[1];
        p[1] = p[2];
        p[2] = tmp;
        p += 4;
      }
      break;
  }
}

inline void endian(const void * in, void * out, size_t size, size_t word_size)
{
  const uint8_t * inP = reinterpret_cast<const uint8_t *>(in);
  uint8_t * outP = reinterpret_cast<uint8_t *>(out);
  
  switch (word_size)
  {
    case 2:
      while (size--) {
        outP[0] = inP[1];
        outP[1] = inP[0];
        inP += 2;
        outP += 2;
      }
      break;
      
    case 4:
      while (size--) {
        outP[0] = inP[3];
        outP[1] = inP[2];
        outP[2] = inP[1];
        outP[3] = inP[0];
        inP += 4;
        outP += 4;
      }
      break;
      
    default: copy(in, out, size * word_size);
  }
}

inline ENDIAN endian_opposite(ENDIAN in)
{
  return MSB == in ? LSB : MSB;
}

}
}
