/*------------------------------------------------------------------------------
 lib/djv_base/vec2_.h

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

namespace djv {
namespace vec
{

#define _VEC2_INIT \
  x(Vec<T, 2>::e[0]), \
  y(Vec<T, 2>::e[1])

template<typename T>
inline Vec2<T>::Vec2() : _VEC2_INIT {}

template<typename T>
inline Vec2<T>::Vec2(const Vec2<T> & in) : _VEC2_INIT { x = in.x; y = in.y; }

template<typename T>
inline Vec2<T>::Vec2(const Vec<T, 2> & in) : _VEC2_INIT
{ x = in.e[0]; y = in.e[1]; }

template<typename T>
inline Vec2<T>::Vec2(T _x, T _y) : _VEC2_INIT { x = _x; y = _y; }

template<typename T>
inline Vec2<T>::Vec2(T in) : _VEC2_INIT { x = y = in; }

template<>
inline void Vec<int, 2>::set(const Vec<int, 2> & in)
{
  Vec<int, 2>::e[0] = in.e[0];
  Vec<int, 2>::e[1] = in.e[1];
}

template<>
inline void Vec<int, 2>::set(int in)
{
  e[0] = e[1] = in;
}

template<>
inline void Vec<int, 2>::zero()
{
  e[0] = e[1] = 0;
}

template<>
inline Vec<int, 2> & Vec<int, 2>::operator = (const Vec<int, 2> & in)
{
  Vec<int, 2>::e[0] = in.e[0];
  Vec<int, 2>::e[1] = in.e[1];
  return *this;
}

#define _VEC2_OP(IN) \
  template<> \
  inline Vec<int, 2> & Vec<int, 2>::operator IN (const Vec<int, 2> & in) \
  { \
    Vec<int, 2>::e[0] IN in.e[0]; \
    Vec<int, 2>::e[1] IN in.e[1]; \
    return *this; \
  }
_VEC2_OP(+=)
_VEC2_OP(-=)
_VEC2_OP(/=)
_VEC2_OP(*=)

#define _VEC2_OP2(IN) \
  template<> \
  inline Vec<int, 2> & Vec<int, 2>::operator IN (int in) \
  { \
    Vec<int, 2>::e[0] IN in; \
    Vec<int, 2>::e[1] IN in; \
    return *this; \
  }
_VEC2_OP2(+=)
_VEC2_OP2(-=)
_VEC2_OP2(/=)
_VEC2_OP2(*=)

}
}

