/*------------------------------------------------------------------------------
 lib/djv_image/gl_data_copy.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_image/gl_data_copy.cxx"

#include "gl_data.h"
#include <djv_image/gl_offscreen.h>

namespace djv {
namespace image {
namespace gl {
namespace data
{

void copy(const image::Data & in, image::Data * out, const Draw & draw,
  void * context_share) throw (Error)
{
  DJV_DBG_OBJ2(_DBG, "copy");
  DJV_DBG_PRN("in = " << in);
  DJV_DBG_PRN("out = " << *out);
  DJV_DBG_PRN("channel = " << draw.channel);

  const Vec2i & size = out->info().size;

  Draw _draw = draw;
  if (out->info().mirror_x) _draw.mirror_x = ! _draw.mirror_x;
  if (out->info().mirror_y) _draw.mirror_y = ! _draw.mirror_y;
  
  offscreen::Pbuffer pbuffer(out->info().size, context_share);
  const Vec2i & pbuffer_size = pbuffer.size();
  Data data;

  //DJV_DBG_GL(glClearColor(0, 1, 0, 0));
  djv::image::Color background(djv::image::pixel::RGB_F32);
  djv::image::color::convert(draw.background, &background);
  DJV_DBG_GL(glClearColor(
    background.get_f32(0), background.get_f32(1), background.get_f32(2), 0.0));
  
  for (int y = 0; y < size.y; y += pbuffer_size.y)
    for (int x = 0; x < size.x; x += pbuffer_size.x)
    {
      const Box2i tile(x, y,
        math::min(pbuffer_size.x, size.x - x),
        math::min(pbuffer_size.y, size.y - y));
      DJV_DBG_PRN("tile = " << tile);
      
      DJV_DBG_GL(glMatrixMode(GL_PROJECTION));
      DJV_DBG_GL(glLoadIdentity());
      DJV_DBG_GL(glOrtho(tile.x, tile.x + tile.w, tile.y, tile.y + tile.h,
        -1.0, 1.0));
      DJV_DBG_GL(glMatrixMode(GL_MODELVIEW));
      DJV_DBG_GL(glLoadIdentity());
      DJV_DBG_GL(glViewport(0, 0, tile.w, tile.h));
      DJV_DBG_GL(glClear(GL_COLOR_BUFFER_BIT));
      
      data.draw(in, tile, _draw);
      data.read(out, tile);
    }
}

}
}
}
}

