/*------------------------------------------------------------------------------
 lib/djv_ui/button_push.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_ui/button_push.cxx"

#include "button.h"
#include <djv_ui/style.h>
#include <FL/Fl.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Button.H>
#include <FL/fl_draw.H>

namespace djv {
namespace ui {
namespace button
{

class Push::Widget : public Fl_Button
{
public:

  Widget() : Fl_Button(0, 0, 0, 0)
  {
    box(FL_THIN_UP_BOX);
    down_box(FL_THIN_DOWN_BOX);
  }
};

Push::Push(const Str & in)
{
  //text_font(style::global()->font_bold());

  Fl_Group::add(_widget = new Widget);
  
  Item::size(layout::FIXED, layout::FIXED);
  
  label(in);
  
  _widget->callback(widget_callback, this);
}

void Push::label(const Str & in)
{
  if (in == _label) return;

  _label = in;
  
  const int size = _label.size() + 1;
  _label_ascii.size(size);
  str::ascii(_label, _label_ascii(), size);
  _widget->label(_label_ascii());
}

void Push::draw()
{
  widget::Widget::draw();
}

void Push::dirty()
{
  DJV_DBG_OBJ2(_DBG, "Push::dirty");
  
  widget::Widget::dirty();

  _widget->labelfont(text_font());
  _widget->labelsize(style::global()->font_size());
  _widget->labelcolor(text_color());
  
  fl_font(text_font(), style::global()->font_size());
  const Vec2i label_size = str_size(_label_ascii());

  size_hint(Vec2i(math::max(label_size.x, style::global()->size_button()),
    label_size.y) +
    style::global()->margin_widget() * 2 +
    Fl::box_dx(_widget->box()) * 2);
}

void Push::geom(const Box2i & in)
{
  widget::Widget::geom(in);
  _widget->resize(in.x, in.y, in.w, in.h);
}

void Push::widget_callback()
{
  sig.emit(true);
}

}
}
}
