/*------------------------------------------------------------------------------
 lib/djv_ui/button_radio.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include "button.h"
#include <djv_ui/style.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/label.h>
#include <FL/Fl.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Round_Button.H>
#include <FL/fl_draw.H>

namespace djv {
namespace ui {
namespace button
{

class Radio::Widget : public Fl_Round_Button, public widget::Base
{
public:

  Widget() : Fl_Round_Button(0, 0, 0, 0) {}

  int handle(int in)
  {
    switch (widget::Base::handle(in))
    {
      case -1: return 0;
      case 1: return 1;
    }
    switch (in)
    {
      // XXX Override FLTK behavior.
    
      case FL_SHORTCUT:
        if (! (shortcut() ? Fl::test_shortcut(shortcut()) : test_shortcut()))
          return 0;
        if (visible_focus() && handle(FL_FOCUS))
          Fl::focus(this);
        value(! value());
        return 1;
    }
    return Fl_Round_Button::handle(in);
  }
  
  void draw()
  {
    Fl_Round_Button::draw();
    widget::Base::draw();
  }

  void dirty()
  {
    widget::Base::dirty();

    labelfont(text_font());
    labelsize(style::global()->font_size());
    labelcolor(text_color());
  
    fl_font(text_font(), style::global()->font_size());
    size_hint(Vec2i(fl_height()) + style::global()->margin_widget() * 2);
  }
  
  void geom(const Box2i & in)
  {
    Fl_Round_Button::resize(in.x, in.y, in.w, in.h);
    layout::Item::geom(in);
  }
  
  const Box2i & geom() const { return widget::Base::geom(); }
  
  void resize(int x, int y, int w, int h) { geom(Box2i(x, y, w, h)); }
  
  int visible() const { return Fl_Round_Button::visible(); }
};

Radio::Radio(const Str & in) : _value(false)
{
  //text_font(style::global()->font_bold());

  Fl_Group::add(_widget = new Widget);
  
  size(layout::FIXED, layout::FIXED);
  align(layout::LEFT, layout::CENTER);
  
  label(in);
  
  _widget->callback(widget_callback, this);
}
  
void Radio::set(bool in)
{
  if (in == _value) return;
  _value = in;
  _widget->value(_value);
  if (_value)
  {
    sig.emit(_value);
    for (uint i = 0; i < _group.size(); ++i)
    if (_group[i] != this) _group[i]->set(false);
  }
}

void Radio::label(const Str & in)
{
  if (in == _label) return;
  _label = in;
  _label_tmp = " " + _label; // XXX Add spacing.
  
  const int size = _label_tmp.size() + 1;
  _label_ascii.size(size);
  str::ascii(_label_tmp, _label_ascii(), size);
  _widget->label(_label_ascii());
}

void Radio::group(const List<Radio *> & in) { _group = in; }

void Radio::draw()
{
  widget::Widget::draw();
}

void Radio::widget_del()
{
  widget::Widget::widget_del();
  _widget->widget_del();
}

void Radio::dirty()
{
  widget::Widget::dirty();

  _widget->labelfont(text_font());
  _widget->labelsize(style::global()->font_size());
  _widget->labelcolor(text_color());
  
  fl_font(text_font(), style::global()->font_size());
  const Vec2i label_size = str_size(_label_ascii());
  
  size_hint(Vec2i(
    math::max(fl_height() + style::global()->margin_widget() + label_size.x,
      style::global()->size_button()),
    label_size.y));
}

void Radio::geom(const Box2i & in)
{
  widget::Widget::geom(in);
  _widget->geom(in);
}

void Radio::widget_callback() { set(_widget->value() ? true : false); }

}
}
}

