/*------------------------------------------------------------------------------
 lib/djv_ui/file_browser_callback.cxx
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include "_file_browser.h"
#include <djv_base/dir.h>
#include <djv_ui/button.h>
#include <djv_ui/menu.h>

namespace djv {
namespace ui {
namespace file_browser
{

void Dialog::widget_callback(const Str & in)
{
  _value = in;
}

void Dialog::widget_action_callback(const Str & in)
{
  File tmp(djv::file::is_path_absolute(in) ? in : (_dir + in));
  if (djv::file::DIR == tmp.type)
  {
    dir(tmp);
  }
  else
  {
    _value = in;
    ok_callback(true);
  }
}

void Dialog::dir_callback(const Str & in) { dir(in); }

void Dialog::browser_callback(const File & in)
{
  _value = in;
  widget_update();
}

void Dialog::browser_action_callback(const File & in)
{
  if (djv::file::DIR == in.type)
  {
    dir(in);
  }
  else
  {
    _value = in;
    widget_update();
    ok_callback(true);
  }
}

void Dialog::columns_callback(const List<int> & in)
{
  _dir_browser->column_list(in);
}

void Dialog::columns_action_callback(int in)
{
  if (in == _sort)
  {
    sort_reverse(! _sort_reverse);
  }
  else
  {
    sort(SORT(in));
  }
}

void Dialog::up_callback(bool)
{
  Dir tmp(_dir);
  tmp.up();
  dir(tmp.get());
}

void Dialog::_up_callback() { up_callback(true); }

void Dialog::prev_callback(bool) { dir(_dir_prev); }
void Dialog::_prev_callback() { prev_callback(true); }

void Dialog::_recent_callback()
{
  dir(_recent[list::find(_menu_widget->menu(), _menu_recent)]);
}

void Dialog::current_callback(bool) { dir(Dir(".").get()); }
void Dialog::_current_callback() { current_callback(true); }

#if ! defined(WIN32)

void Dialog::home_callback(bool) { dir(djv::dir::home()); }
void Dialog::_home_callback() { home_callback(true); }

#endif

void Dialog::desktop_callback(bool) { dir(djv::dir::desktop()); }
void Dialog::_desktop_callback() { desktop_callback(true); }

#if defined(WIN32)

void Dialog::drive_callback(bool)
{
  const List<Str> drive_list = djv::dir::drive_list();
  const int i = list::find(_menu_widget->menu(), _menu_drive);
  if (i >= 0 && i < int(drive_list.size())) dir(drive_list[i]);
}

void Dialog::_drive_callback() { drive_callback(true); }

#else

void Dialog::root_callback(bool) { dir(djv::dir::root()); }
void Dialog::_root_callback() { root_callback(true); }

#endif

void Dialog::reload_callback(bool) { dir_update(); }
void Dialog::_reload_callback() { reload_callback(true); }

void Dialog::_images_callback() { images(! _images);  }
void Dialog::_seq_callback() { seq(! _seq); }
void Dialog::_hidden_callback() { hidden(! _hidden); }

void Dialog::_sort_callback()
{ sort(SORT(list::find(_menu_widget->menu(), _menu_sort))); }
void Dialog::_sort_reverse_callback() { sort_reverse(! _sort_reverse); }
void Dialog::_sort_dirs_callback() { sort_dirs(! _sort_dirs); }

void Dialog::_bookmark_callback()
{
  dir(_bookmarks[list::find(_menu_widget->menu(), _menu_bookmark)]);
}

void Dialog::_bookmark_add_callback()
{
  _bookmarks += _dir;
  menu_update();
}

void Dialog::bookmark_del_callback(int in)
{
  if (in >= int(_bookmarks.size())) return;
  List<Str>::iterator i = _bookmarks.begin();
  for (int j = 0; j < in; ++j, ++i) ;
  _bookmarks.erase(i);
  menu_update();
}

void Dialog::_bookmark_del_callback()
{
  if (! _bookmarks.size()) return;
  ui::dialog::choice("Delete bookmark:", _bookmarks,
    this, bookmark_del_callback);
}

void Dialog::bookmark_del_all_callback(bool)
{
  _bookmarks.clear();
  menu_update();
}

void Dialog::_bookmark_del_all_callback()
{
  if (_bookmarks.size())
    dialog::question("Delete all bookmarks?", this, bookmark_del_all_callback);
}

void Dialog::pin_callback(bool) { pin(_pin_button->get()); }

void Dialog::ok_callback(bool)
{
  sig.emit(get());
  
  if (! _pin) hide();
  
  ui::recent(dir(), &_recent);
  menu_update();
}

void Dialog::close_callback(bool) { hide(); }

}
}
}
