/*------------------------------------------------------------------------------
 lib/djv_ui/file_browser_menu.cxx
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_ui/file_browser_menu.cxx"

#include "_file_browser.h"
#include <djv_base/dir.h>
#include <djv_ui/menu.h>

namespace djv {
namespace ui {
namespace file_browser
{

void Dialog::menu_update()
{
  DJV_DBG_OBJ2(_DBG, "Dialog::menu_update");
  DJV_DBG_PRN("images = " << _images);
  DJV_DBG_PRN("seq = " << _seq);
  DJV_DBG_PRN("hidden = " << _hidden);
  DJV_DBG_PRN("sort = " << _sort);
  DJV_DBG_PRN("sort reverse = " << _sort_reverse);
  DJV_DBG_PRN("sort dirs = " << _sort_dirs);
  
  _menu_widget->menu_del();
  
  List<Str> tmp;
  
  // Directory menu.

  _menu_widget->menu_add("&Directory", 0, 0, 0, menu::SUB_MENU);
  _menu_widget->menu_add("&Up", '\\', _up_callback, this);
  _menu_widget->menu_add("&Previous", FL_BackSpace, _prev_callback, this);
  _menu_widget->menu_add("Re&cent", 0, 0, 0, menu::SUB_MENU | menu::DIVIDER);
  for (uint i = 0; i < _recent.size(); ++i)
    _menu_recent += _menu_widget->menu_add(
      menu::list_name(_recent[i], i), 0,
      _recent_callback, this);
  _menu_widget->menu_end();

  _menu_widget->menu_add("C&urrent", '.', _current_callback, this);
#if ! defined(WIN32)
  _menu_widget->menu_add("&Home", ',', _home_callback, this);
#endif
  _menu_widget->menu_add("Des&ktop", 0, _desktop_callback, this);
#if defined(WIN32)
  _menu_widget->menu_add("Dri&ves", 0, 0, 0, menu::SUB_MENU | menu::DIVIDER);
  tmp = djv::dir::drive_list();
  for (uint i = 0; i < tmp.size(); ++i)
    _menu_drive += _menu_widget->menu_add(
      menu::list_name(tmp[i], i), 0,
      _drive_callback, this);
  _menu_widget->menu_end();
#else
  _menu_widget->menu_add("Roo&t", '/', _root_callback, this, menu::DIVIDER);
#endif

  _menu_widget->menu_add("&Reload", 'r', _reload_callback, this);
  _menu_widget->menu_end();
  
  // View menu.

  _menu_widget->menu_add("&View", 0, 0, 0, menu::SUB_MENU);
  
  _menu_widget->menu_add("&Images", 0, _images_callback, this,
    menu::TOGGLE | (_images ? menu::VALUE : 0));
  _menu_widget->menu_add("Se&quences", 'q', _seq_callback, this,
    menu::TOGGLE | (_seq ? menu::VALUE : 0));
  _menu_widget->menu_add("Hidde&n", 'n', _hidden_callback, this,
    menu::TOGGLE | (_hidden ? menu::VALUE : 0) | menu::DIVIDER);
  
  _menu_widget->menu_add("&Sort", 0, 0, 0, menu::SUB_MENU);
  _menu_sort.clear();
  tmp = label_sort();
  for (uint i = 0; i < tmp.size(); ++i)
    _menu_sort += _menu_widget->menu_add(
      ui::menu::list_name(tmp[i], i), '1' + i,
      _sort_callback, this,
      menu::RADIO | (int(i) == _sort ? menu::VALUE : 0));
  _menu_widget->menu_end();
  _menu_widget->menu_add("&Reverse Sort", '`', _sort_reverse_callback, this,
    menu::TOGGLE | (_sort_reverse ? menu::VALUE : 0));
  _menu_widget->menu_add("Sort &Directories", 'd',
    _sort_dirs_callback, this, menu::TOGGLE | (_sort_dirs ? menu::VALUE : 0));
  
  _menu_widget->menu_end();
  
  // Bookmarks menu.

  _menu_widget->menu_add("&Bookmarks", 0, 0, 0, menu::SUB_MENU);
  _menu_widget->menu_add("&Add", 'b', _bookmark_add_callback, this);
  _menu_widget->menu_add("&Delete", 0, _bookmark_del_callback, this);
  _menu_widget->menu_add("Delete A&ll", 0, _bookmark_del_all_callback, this,
    menu::DIVIDER);
  
  _menu_bookmark.clear();
  for (uint i = 0; i < _bookmarks.size(); ++i)
    _menu_bookmark += _menu_widget->menu_add(
      menu::list_name(_bookmarks[i], i), i < 12 ? (FL_F + i + 1) : 0,
      _bookmark_callback, this);
  _menu_widget->menu_end();
  
  _menu_widget->menu_end();
}

}
}
}
