/*------------------------------------------------------------------------------
 lib/djv_ui/group_box.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_ui/group_box.cxx"

#include "group.h"
#include <djv_ui/style.h>
#include <djv_ui/layout_row.h>
#include <FL/Fl_Group.H>
#include <FL/fl_draw.H>

namespace djv {
namespace ui {
namespace group
{

Box::Box(const Str & label) : _label(label), _label_widget(0)
{
  //text_font(style::global()->font_bold());
  
  new layout::Vertical(this);
}

void Box::label(const Str & in)
{
  if (in == _label) return;
  _label = in;
  redraw();
}

void Box::label_widget(Widget * in)
{
  if (_label_widget)
  {
    remove(_label_widget);
    _label_widget->widget_del();
  }
  
  _label_widget = in;
  
  if (_label_widget)
  {
    add(_label_widget);
    _label_widget->ui::layout::Item::parent(this);
    //_label_widget->text_font(style::global()->font_bold());
  }
  
  update();
  redraw();
}

void Box::draw()
{
  const Box2i & geom = this->geom();
  const int margin = style::global()->margin_widget();

  fl_font(text_font(), style::global()->font_size());
  
  // Draw box.
  
  //fl_color(fl_darker(FL_BACKGROUND_COLOR));
  
  /*fl_rect(
    geom.x, geom.y + _size_label.y / 2,
    geom.w, geom.h - _size_label.y / 2);*/
  
  /*fl_line(
    geom.x, geom.y + _size_label.y / 2,
    geom.x + margin, geom.y + _size_label.y / 2);
  fl_line(
    geom.x + _size_label.x + margin * 3, geom.y + _size_label.y / 2,
    geom.x + geom.w - 1, geom.y + _size_label.y / 2);
  fl_line(
    geom.x, geom.y + _size_label.y / 2,
    geom.x, geom.y + geom.h - 1);
  fl_line(
    geom.x + geom.w - 1, geom.y + _size_label.y / 2,
    geom.x + geom.w - 1, geom.y + geom.h - 1);
  fl_line(
    geom.x, geom.y + geom.h - 1,
    geom.x + geom.w - 1, geom.y + geom.h - 1);*/
  
  fl_draw_box(FL_ENGRAVED_FRAME, geom.x,  geom.y + _size_label.y / 2,
    geom.w, geom.h - _size_label.y / 2, FL_BACKGROUND_COLOR);
  
  // Draw label.
  
  fl_color(FL_BACKGROUND_COLOR);
  //fl_color(FL_BLUE);
  fl_rectf(geom.x + 2 + margin, geom.y, _size_label.x + margin * 2,
    _size_label.y);
  
  if (_label_widget)
  {
    _label_widget->damage(FL_DAMAGE_ALL);
  }
  else
  {
    fl_color(FL_FOREGROUND_COLOR);
    fl_draw(_label.c_str(),
      geom.x + 2 + margin * 2, geom.y + fl_height() - fl_descent());
  }

  widget::Widget::draw();
}

void Box::widget_del()
{
  if (_label_widget) _label_widget->widget_del();
  widget::Widget::widget_del();
}

void Box::dirty()
{
  DJV_DBG_OBJ2(_DBG, "Box::dirty");
  
  widget::Widget::dirty();

  if (_label_widget) _label_widget->dirty();
  
  fl_font(text_font(), style::global()->font_size());

  const Vec2i size = layout() ? layout()->size_hint() : Vec2i();
  DJV_DBG_PRN("size = " << size);

  _size_label = _label_widget ? _label_widget->size_hint() :
    Vec2i(str_width(_label), fl_height());
  DJV_DBG_PRN("label size = " << _size_label);
  
  size_hint(Vec2i(
    math::max(_size_label.x + style::global()->margin_widget() * 2, size.x) +
      style::global()->margin_widget() * 2 + 2 * 2,
    _size_label.y + size.y + style::global()->margin_widget() + 2));
}

void Box::geom(const Box2i & in)
{
  DJV_DBG_OBJ2(_DBG, "Box::geom");
  DJV_DBG_PRN("in = " << in);
  
  //Fl_Group::resize(in.x, in.y, in.w, in.h);
  Fl_Widget::resize(in.x, in.y, in.w, in.h);
  layout::Item::geom(in);
  
  const int margin = style::global()->margin_widget();

  if (_label_widget)
    _label_widget->geom(Box2i(in.x + 2 + margin * 2, in.y, _size_label.x,
      _size_label.y));
  
  if (layout())
    layout()->geom(Box2i(in.x + 2, in.y + _size_label.y + margin, in.w - 2 * 2,
      in.h - _size_label.y - margin - 2));
}

}
}
}

