/*------------------------------------------------------------------------------
 lib/djv_ui/image.h
 
 Image Widget
 ------------
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_IMAGE_H
#define DJV_UI_IMAGE_H

#include <djv_image/color.h>
#include <djv_image/profile.h>
#include <djv_image/gl_data.h>
#include <djv_ui/gl.h>

namespace djv {
namespace ui {
namespace image
{

//------------------------------------------------------------------------------
// Options

enum SCALE { SCALE_NONE, SCALE_16_9, SCALE_1_85, SCALE_2_35 };
enum ROTATE { ROTATE_0, ROTATE_90, ROTATE_180, ROTATE_270 };
using djv::image::Profile;
using djv::image::gl::data::Display;
using djv::image::gl::data::CHANNEL;
using djv::image::gl::data::FILTER;

//------------------------------------------------------------------------------
// Widget

class Widget : public gl::Widget
{
public:

  Widget();
  virtual ~Widget() {}
  
  // Data.

  virtual void set(const djv::image::data::Data *);
  const djv::image::data::Data * get() const { return _data; }

  // View.
  
  virtual void view(const Vec2i &);
  const Vec2i & view() const { return _view; }
  
  virtual void zoom(double);
  virtual void zoom(double, const Vec2i & focus);
  double zoom() const { return _zoom; }

  virtual void view_zero();
  virtual void view_center();
  virtual void view_fit();

  Signal<bool> sig;
  
  // Options.
  
  void mirror(bool x, bool y);
  void scale(SCALE);
  void rotate(ROTATE);
  void profile(const Profile &);
  void display(const Display &);
  void channel(CHANNEL);
  void filter(FILTER);
  
  bool mirror_x() const { return _mirror_x; }
  bool mirror_y() const { return _mirror_y; }
  SCALE scale() const { return _scale; }
  ROTATE rotate() const { return _rotate; }
  const Profile & profile() const { return _profile; }
  const Display & display() const { return _display; }
  CHANNEL channel() const { return _channel; }
  FILTER filter() const { return _filter; }

  // Widget.

  virtual void background(const djv::image::Color &);
  const djv::image::Color & background() const { return _background; }
  
  virtual void draw();
  
  // Functions.

  Vec2i screen_size() const;

private:

  const djv::image::data::Data * _data;
  djv::image::gl::data::Data _gl_data;
  Vec2i _view;
  double _zoom;
  bool _mirror_x, _mirror_y;
  SCALE _scale;
  ROTATE _rotate;
  Profile _profile;
  Display _display;
  CHANNEL _channel;
  FILTER _filter;
  djv::image::Color _background;
};

//------------------------------------------------------------------------------
// Functions

double scale(SCALE);
double rotate(ROTATE);

Vec2i size(const Vec2i &, SCALE, ROTATE = ROTATE(0));
Vec2i position(const Vec2i &, const Vec2i & size, ROTATE);

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_scale();
const List<Str> & label_rotate();
using djv::image::gl::data::label_channel;
using djv::image::gl::data::label_filter;

}
}
}

#endif

