/*------------------------------------------------------------------------------
 lib/djv_ui/image_io_lut.cxx
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_ui/image_io_builtin.cxx"

#include "image_io_builtin.h"
#include <djv_ui/style.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/group.h>
#include <djv_ui/label.h>
#include <djv_ui/choice.h>

namespace djv {
namespace ui {
namespace image_io
{
using namespace djv::image::io;

Lut_Load::Lut_Load() : _plugin(0)
{
  // Widgets.
  
  group::Box * spec_group = new group::Box;
  _spec_widget = new choice::Widget(lut::label_spec());
  _spec_widget->label(new label::Widget("Spec:"));
  spec_group->label_widget(_spec_widget);
  ui::label::Text * spec_text = new ui::label::Text("File specification.");
  
  group::Box * type_group = new group::Box;
  _type_widget = new choice::Widget(lut::label_type());
  _type_widget->label(new label::Widget("Type:"));
  type_group->label_widget(_type_widget);
  ui::label::Text * type_text = new ui::label::Text("Pixel type.");
  
  // Layout.

  layout::Vertical * layout = new layout::Vertical(this);
  layout->spacing(ui::style::global()->spacing_large());

  layout->add(spec_group);
  layout::Vertical * group_layout = new layout::Vertical(spec_group);
  group_layout->add(spec_text);
  
  layout->add(type_group);
  group_layout = new layout::Vertical(type_group);
  group_layout->add(type_text);
  
  layout->add_stretch();
  
  // Initialize.
  
  plugin_update();
  widget_update();
  
  // Callbacks.
  
  _spec_widget->sig.set(this, spec_callback);
  _type_widget->sig.set(this, type_callback);
}

void Lut_Load::set(djv::image::io::Plugin * in)
{
  _plugin = in;
  widget_update();
}

void Lut_Load::spec_callback(int in)
{
  if (! _plugin) return;
  _options.spec = lut::SPEC(in);
  plugin_update();
  sig.emit(_plugin);
}

void Lut_Load::type_callback(int in)
{
  if (! _plugin) return;
  _options.type = lut::TYPE(in);
  plugin_update();
  sig.emit(_plugin);
}

void Lut_Load::plugin_update()
{
  callbacks(false);
  
  if (_plugin)
  {
    List<Str> tmp;
    tmp = str::serialize(_options.spec, lut::label_spec());
    _plugin->option(_plugin->option_list()[lut::Load::OPTION_SPEC], &tmp);
    tmp = str::serialize(_options.type, lut::label_type());
    _plugin->option(_plugin->option_list()[lut::Load::OPTION_TYPE], &tmp);
  }
  
  callbacks(true);
}

void Lut_Load::widget_update()
{
  callbacks(false);
  
  if (_plugin)
  {
    List<Str> tmp;
    tmp = _plugin->option(_plugin->option_list()[lut::Load::OPTION_SPEC]);
    str::serialize(&tmp, &_options.spec, lut::label_spec());
    tmp = _plugin->option(_plugin->option_list()[lut::Load::OPTION_TYPE]);
    str::serialize(&tmp, &_options.type, lut::label_type());
  }
  
  _spec_widget->set(_options.spec);
  _type_widget->set(_options.type);
  
  callbacks(true);
}

Lut_Save::Lut_Save() : _plugin(0)
{
  // Widgets.
  
  group::Box * spec_group = new group::Box;
  _spec_widget = new choice::Widget(lut::label_spec());
  _spec_widget->label(new label::Widget("Spec:"));
  spec_group->label_widget(_spec_widget);
  ui::label::Text * spec_text = new ui::label::Text("File specification.");
  
  // Layout.

  layout::Vertical * layout = new layout::Vertical(this);
  layout->spacing(ui::style::global()->spacing_large());

  layout->add(spec_group);
  layout::Vertical * group_layout = new layout::Vertical(spec_group);
  group_layout->add(spec_text);
  
  layout->add_stretch();
  
  // Initialize.
  
  plugin_update();
  widget_update();
  
  // Callbacks.
  
  _spec_widget->sig.set(this, spec_callback);
}

void Lut_Save::set(djv::image::io::Plugin * in)
{
  _plugin = in;
  widget_update();
}

void Lut_Save::spec_callback(int in)
{
  if (! _plugin) return;
  _options.spec = lut::SPEC(in);
  plugin_update();
  sig.emit(_plugin);
}

void Lut_Save::plugin_update()
{
  callbacks(false);
  
  if (_plugin)
  {
    List<Str> tmp;
    tmp = str::serialize(_options.spec, lut::label_spec());
    _plugin->option(_plugin->option_list()[lut::Save::OPTION_SPEC], &tmp);
  }
  
  callbacks(true);
}

void Lut_Save::widget_update()
{
  callbacks(false);
  
  if (_plugin)
  {
    List<Str> tmp;
    tmp = _plugin->option(_plugin->option_list()[lut::Save::OPTION_SPEC]);
    str::serialize(&tmp, &_options.spec, lut::label_spec());
  }
  
  _spec_widget->set(_options.spec);
  
  callbacks(true);
}

}
}
}

