/*------------------------------------------------------------------------------
 lib/djv_ui/image_io_tga.cxx
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_ui/image_io_builtin.cxx"

#include "image_io_builtin.h"
#include <djv_ui/style.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/group.h>
#include <djv_ui/label.h>
#include <djv_ui/choice.h>

namespace djv {
namespace ui {
namespace image_io
{
using namespace djv::image::io;

Tga_Save::Tga_Save() : _plugin(0)
{
  DJV_DBG_OBJ2(_DBG, "Tga_Save::Tga_Save");
  
  // Widgets.
  
  group::Box * compress_group = new group::Box;
  _compress_widget = new choice::Widget(tga::label_compress());
  _compress_widget->label(new label::Widget("Compress:"));
  compress_group->label_widget(_compress_widget);
  ui::label::Text * compress_text = new ui::label::Text("File compression.");
  
  // Layout.

  layout::Vertical * layout = new layout::Vertical(this);
  layout->spacing(ui::style::global()->spacing_large());

  layout->add(compress_group);
  layout::Vertical * group_layout = new layout::Vertical(compress_group);
  group_layout->add(compress_text);
  
  layout->add_stretch();
  
  // Initialize.

  plugin_update();
  widget_update();
  
  // Callbacks.
  
  _compress_widget->sig.set(this, compress_callback);
}

Tga_Save::~Tga_Save()
{
  DJV_DBG_OBJ2(_DBG, "Tga_Save::~Tga_Save");
}

void Tga_Save::set(djv::image::io::Plugin * in)
{
  _plugin = in;
  widget_update();
}

void Tga_Save::compress_callback(int in)
{
  if (! _plugin) return;
  _options.compress = tga::COMPRESS(in);
  plugin_update();
  sig.emit(_plugin);
}

void Tga_Save::plugin_update()
{
  callbacks(false);
  
  if (_plugin)
  {
    List<Str> tmp;
    tmp = str::serialize(_options.compress, tga::label_compress());
    _plugin->option(_plugin->option_list()[tga::Save::OPTION_COMPRESS], &tmp);
  }
  
  callbacks(true);
}

void Tga_Save::widget_update()
{
  callbacks(false);
  
  if (_plugin)
  {
    List<Str> tmp;
    tmp = _plugin->option(_plugin->option_list()[tga::Save::OPTION_COMPRESS]);
    str::serialize(&tmp, &_options.compress, tga::label_compress());
  }
  
  _compress_widget->set(_options.compress);
  
  callbacks(true);
}

}
}
}
