/*------------------------------------------------------------------------------
 lib/djv_ui/layout.h
 
 Widget Layout
 -------------

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_LAYOUT_H
#define DJV_UI_LAYOUT_H

#include <djv_base/base.h>
#include <djv_base/box.h>

namespace djv {
namespace ui {
namespace layout
{

//------------------------------------------------------------------------------
// Layout Item

enum ITEM { WIDGET, LAYOUT, SPACER };
enum SIZE { FIXED, STRETCH };
enum ALIGN { LEFT, CENTER, RIGHT };

class Item
{
public:

  Item(ITEM);
  virtual ~Item() = 0;
  
  ITEM item() const { return _item; }

  virtual void dirty();

  // Layout.
  
  virtual void update();
  virtual void geom(const Box2i &);
  virtual const Box2i & geom() const { return _geom; }
  virtual const Vec2i & size_hint() const { return _size_hint; }
  virtual const Vec2i & size_min() const { return _size_hint; }
  virtual int visible() const { return true; }

  // Size.

  void size(SIZE, SIZE);
  void align(ALIGN, ALIGN);
  const SIZE * size() const { return _size; }
  const ALIGN * align() const { return _align; }
  
  // XXX Cheesy way to align items.

  void group(const List<Item *> &);
  const List<Item *> & group() const { return _group; }
  virtual Item * group_align() { return this; }
  virtual Item * label_align() const { return 0; }

  // Parent.

  virtual void parent(Item *);
  Item * parent() const { return _parent; }
  
  // Debugging.
  
  virtual Str dbg_name() const { return "djv::ui::layout::Item"; }

protected:

  void size_hint(const Vec2i &);

private:

  ITEM _item;
  Box2i _geom;
  Vec2i _size_hint;
  SIZE _size [2];
  ALIGN _align [2];
  List<Item *> _group;
  Item * _parent;
};

//------------------------------------------------------------------------------
// Layout Spacer

class Spacer : public Item
{
public:

  Spacer() : Item(SPACER) {}
};

//------------------------------------------------------------------------------
// Layout

class Layout : public Item
{
public:

  Layout(Item * parent = 0);
  virtual ~Layout() = 0;

  // Items.
  
  virtual void add(Item *);
  virtual void insert(Item *, int);
  virtual Item * remove(int);
  virtual Item * remove(Item *);
  virtual void del(int);
  virtual void del(Item *);
  virtual void del();
  const List<Item *> & list() const { return _list; }

  // Layout.

  void spacing(int);
  void margin(int);
  int spacing() const;
  int margin() const;

  virtual void dirty();
  virtual int visible() const;
  
  // Debugging.
  
  virtual Str dbg_name() const { return "djv::ui::layout::Layout"; }
  void dbg(int indent = 0);

private:

  List<Item *> _list;
  int _spacing, _margin;
};

//------------------------------------------------------------------------------
// Functions

void group(const List<Item *> &);

}
}
}

#endif

