/*------------------------------------------------------------------------------
 lib/djv_ui/num_.h

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include <FL/Fl.H>

namespace djv {
namespace ui {
namespace num
{

template<class T>
inline Base<T>::Base() :
  _value(T(0)),
  _default_valid(false), _default_value(T(0)),
  _min(T(0)), _max(T(0)),
  _clamp(true),
  _inc_small(T(0)), _inc_large(T(0))
{}

template<class T>
inline Base<T>::~Base() {}

template<class T>
inline void Base<T>::set(T in)
{
  const T tmp = _clamp ? math::clamp(in, _min, _max) : in;
  if (tmp == _value) return;
  
  _value = tmp;
  
  sig.emit(_value);

  value_update();
}

template<class T>
inline void Base<T>::range(T min, T max)
{
  if (min == _min && max == _max) return;
  
  _min = min;
  _max = max;
  
  set(_value);
  
  range_update();
}
  
template<class T>
inline void Base<T>::inc(T inc_small, T inc_large)
{
  if (inc_small == _inc_small && inc_large == _inc_large) return;
  
  _inc_small = inc_small;
  _inc_large = inc_large;
  
  range_update();
}

template<class T>
inline void Base<T>::inc_small_action() { set(_value + _inc_small); }
template<class T>
inline void Base<T>::inc_large_action() { set(_value + _inc_large); }
template<class T>
inline void Base<T>::dec_small_action() { set(_value - _inc_small); }
template<class T>
inline void Base<T>::dec_large_action() { set(_value - _inc_large); }

template<class T>
inline void Base<T>::clamp(bool in)
{
  _clamp = in;
}
  
template<class T>
inline void Base<T>::default_valid(bool in) { _default_valid = in; }

template<class T>
inline void Base<T>::default_value(T in)
{
  _default_value = in;
  default_valid(true);
}

template<class T>
inline int Base<T>::handle(int in)
{
  switch (in)
  {
    case FL_KEYBOARD:
      switch (Fl::event_key()) {
        case FL_Up: inc_small_action(); return 1;
        case FL_Down: dec_small_action(); return 1;
        case FL_Page_Up: inc_large_action(); return 1;
        case FL_Page_Down: dec_large_action(); return 1;
      }
      break;
  }
  return 0;
}

}
}
}

