/*------------------------------------------------------------------------------
 lib/djv_ui/widget.h
 
 Widget
 ------
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_WIDGET_H
#define DJV_UI_WIDGET_H

#include <djv_ui/base.h>
#include <djv_ui/layout.h>
#include <FL/Fl_Group.H>

namespace djv {
namespace ui {
namespace widget
{

//------------------------------------------------------------------------------
// Widget Base

class Base : public layout::Item, public Callback_Base
{
public:

  Base();
  virtual ~Base() = 0;
  
  // Widget.

  virtual void text_font(const Fl_Font &);
  virtual void text_color(const Fl_Color &);
  virtual const Fl_Font & text_font() const { return _text_font; }
  virtual const Fl_Color & text_color() const { return _text_color; }

  virtual void tooltip(const Str &);
  virtual const Str & tooltip() const { return _tooltip; }
  
  virtual int handle(int) = 0;
  virtual void draw() = 0;

  // XXX

  virtual void widget_del();
  
  // Layout.

  virtual void dirty();
  virtual void layout(layout::Layout *);
  virtual void geom(const Box2i &) = 0;
  virtual void resize(int, int, int, int) = 0;
  virtual const Box2i & geom() const { return Item::geom(); }
  virtual layout::Layout * layout() const { return _layout.get(); }
  virtual int visible() const = 0;
  
  // Debugging.

  virtual Str dbg_name() const { return "djv::ui::widget::Base"; }

private:

  Fl_Font _text_font;
  Fl_Color _text_color;
  Str _tooltip;
  std::auto_ptr<layout::Layout> _layout;
};

//------------------------------------------------------------------------------
// Widget

class Widget : public Fl_Group, public Base
{
public:

  Widget();
  virtual ~Widget();
  
  // Widget.
  
  virtual void tooltip(const Str &);
  virtual const Str & tooltip() const { return Base::tooltip(); }
  
  virtual int handle(int);
  virtual void draw();
  
  // Layout.

  virtual void dirty();
  virtual void geom(const Box2i &);
  virtual void resize(int x, int y, int w, int h) { geom(Box2i(x, y, w, h)); }
  virtual const Box2i & geom() const { return Base::geom(); }
  virtual int visible() const { return _visible; }

  void size(layout::SIZE x, layout::SIZE y) { Base::size(x, y); }
  void align(layout::ALIGN x, layout::ALIGN y) { Base::align(x, y); }
  const layout::SIZE * size() const { return Base::size(); }
  const layout::ALIGN * align() const { return Base::align(); }
  
  // Debugging.
  
  virtual Str dbg_name() const { return "djv::ui::widget::Widget"; }
  
private:
  
  mem::Buffer<char> _tooltip_ascii;
  bool _visible;
};

}
}
}

#endif

