/*------------------------------------------------------------------------------
 plugin/djv_tiff/ui.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "plugin/djv_tiff/ui.cxx"

#include "ui.h"
#include <djv_ui/style.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/group.h>
#include <djv_ui/label.h>
#include <djv_ui/choice.h>

extern "C"
{
DJV_PLUGIN_EXPORT plugin::Plugin * djv_ui_image_save()
  { return new Save_Widget; }
}

Save_Widget::Save_Widget() : _plugin(0)
{
  DJV_DBG_OBJ2(_DBG, "Save_Widget::Save_Widget");

  // Widgets.
  
  ui::group::Box * compress_group = new ui::group::Box;
  _compress_widget = new ui::choice::Widget(label_compress());
  _compress_widget->label(new ui::label::Widget("Compress:"));
  compress_group->label_widget(_compress_widget);
  ui::label::Text * compress_text = new ui::label::Text("File compression.");
  
  // Layout.
  
  ui::layout::group(List<ui::layout::Item *>() << _compress_widget);
  
  ui::layout::Vertical * layout = new ui::layout::Vertical(this);
  layout->spacing(ui::style::global()->spacing_large());

  layout->add(compress_group);
  ui::layout::Vertical * group_layout =
    new ui::layout::Vertical(compress_group);
  group_layout->add(compress_text);

  layout->add_stretch();
  
  // Initialize.
  
  plugin_update();
  widget_update();
  
  // Callbacks.
  
  _compress_widget->sig.set(this, compress_callback);
}

void Save_Widget::set(image::io::Plugin * in)
{
  _plugin = in;
  widget_update();
}

void Save_Widget::compress_callback(int in)
{
  if (! _plugin) return;
  _options.compress = COMPRESS(in);
  plugin_update();
  sig.emit(_plugin);
}

void Save_Widget::plugin_update()
{
  callbacks(false);

  if (_plugin)
  {
    List<Str> tmp;
    tmp = str::serialize(_options.compress, label_compress());
    _plugin->option(_plugin->option_list()[::Save::OPTION_COMPRESS], &tmp);
  }

  callbacks(true);
}

void Save_Widget::widget_update()
{
  callbacks(false);
  
  if (_plugin)
  {
    List<Str> tmp;
    tmp = _plugin->option(_plugin->option_list()[::Save::OPTION_COMPRESS]);
    str::serialize(&tmp, &_options.compress, label_compress());
  }
  
  _compress_widget->set(_options.compress);
  
  callbacks(true);
}

