/*------------------------------------------------------------------------------
 plugin/djv_vlut/save.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "plugin/djv_vlut/save.cxx"

#include "plugin.h"
#include <FL/x.h>

extern "C"
{
DJV_PLUGIN_EXPORT plugin::Plugin * djv_image_save() { return new Save; }
}

void Save::open(const File &, const image::Info &, const image::Tag &,
  const image::Time &) throw (Error)
{
  DJV_DBG_OBJ2(_DBG, "Save::open");
  DJV_DBG_PRN("in = " << in);

  // Information.
  
  int size = 0;
  
#if defined(DJV_VLUT_XORG)

  if (! XF86VidModeGetGammaRampSize(fl_display, fl_screen, &size))
    throw Error(name(), image::io::error_open);

#elif defined(DJV_VLUT_XSGI)

  int maps = 0, precision = 0, attr = 0;
  if (! XSGIvcQueryScreenGammaMaps(fl_display, fl_screen, &maps))
    throw Error(name(), image::io::error_open);
  if (maps <= 0)
    throw Error(name(), image::io::error_open);
  if (! XSGIvcQueryGammaMap(fl_display, fl_screen, 0,
    &size, &precision, &attr)
  )
    throw Error(name(), image::io::error_open);
  if (attr & XSGIVC_GMWriteLock)
    throw Error(name(), image::io::error_open);

#elif defined(DJV_VLUT_WIN32)

  size = 256;

#endif

  DJV_DBG_PRN("vlut size = " << size);
  
  _info = image::Info();
  _info.size = Vec2i(size, 1);
  _info.pixel = image::pixel::RGB_U16;
  DJV_DBG_PRN("info = " << _info);
  
  _image.set(_info);
  _tmp.set(_info);
}

void Save::set(const image::Image & in, int) throw (Error)
{
  DJV_DBG_OBJ2(_DBG, "Save::set");
  DJV_DBG_PRN("in = " << in);

  // Write.
  
  const image::Data * p = &in;
  if (in.info() != _info)
  {
    DJV_DBG_PRN("convert = " << _image);
    _image.zero();
    image::gl::data::Draw draw;
    draw.scale = vec::to_float(_image.size());
    image::gl::data::copy(in, &_image, draw);
    p = &_image;
  }
  
  image::data::planar_deinterleave(*p, &_tmp);
  image::pixel::U16_T * data_p =
    reinterpret_cast<image::pixel::U16_T *>(_tmp.data());
  
#if defined(DJV_DBG)
  //for (int i = 0; i < _info.size.x; ++i)
  //  DJV_DBG_PRN(i << " = " << data_p[i]);
#endif
  
#if defined(DJV_VLUT_XORG)

  if (! XF86VidModeSetGammaRamp(fl_display, fl_screen, _info.size.x,
    data_p, data_p + _info.size.x, data_p + _info.size.x * 2))
  {
    throw Error(name(), image::io::error_write);
  }
  
#elif defined(DJV_VLUT_XSGI)

  // XXX
  
  XSGIvcStoreGammaColors16(fl_display, fl_screen, 0, _info.size.x,
    XSGIVC_MComponentRed | XSGIVC_MComponentGreen | XSGIVC_MComponentBlue,
    data_p);

#elif defined(DJV_VLUT_WIN32)

  ::SetDeviceGammaRamp(handle()->display, data_p);

#endif
}

