//#define DJV_DBG

#include <djv_base/file.h>
#include <djv_image/io.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/window.h>
#include <djv_ui/image.h>
#include <djv_ui/app.h>
#include <FL/Fl.H>

namespace
{
using namespace djv;

class App : public ui::App
{
public:

  App(int argc, char ** argv);
  ~App();
  
  void update();

private:

  DJV_CALLBACK(App, exit_callback, bool) { exit(); }

  File _file;
  image::io::Load * _load;
  image::Info _info;
  image::Time _time;
  ui::window::Widget * _window;
  ui::image::Widget * _widget;
  int _frame;
};

void update_callback(void *);

App::App(int argc, char ** argv) :
  ui::App("image_play", argc, argv),
  _load(0),
  _window(0),
  _frame(0)
{
  if (argc != 2)
  {
    print("Usage: image_play (input)");
    image::App::exit(app::EXIT_ERROR);
    return;
  }
  
  _file.set(argv[1]);
  if (_file.seq_valid()) _file.type = file::SEQ;
  try
  {
    _load = image::io::load(_file, &_info, 0, &_time);
  }
  catch (Error in)
  {
    error(str::Format("%1: %2").arg(_file).arg(in));
    image::App::exit(app::EXIT_ERROR);
    return;
  }
  
  // Widgets.
  
  _window = new ui::window::Widget(name());
  _widget = new ui::image::Widget;
  
  // Layout.
  
  ui::layout::Vertical * layout = new ui::layout::Vertical(_window);
  layout->margin(0);
  layout->add(_widget);
  layout->stretch(_widget);
  
  // Initialize.
  
  _frame = _time.start;
  //update();
  Fl::add_idle(update_callback, this);

  //_widget->zoom(0.5);
  
  _window->size(_info.size);
  _window->show();

  // Callbacks.
  
  _window->close_sig.set(this, exit_callback);

  valid(true);
}

App::~App()
{
  if (_window) _window->widget_del();
  if (_load) delete _load;
}

void App::update()
{
  DJV_DBG_OBJ("App::update");

  static djv::time::Timer t;
  t.check();
  print(str::Format("FPS = %1").arg(t.fps()));
  t.start();
  
  /*for (int i = 0; i < 3; ++i)
  {
    try { djv::io::Io io; io.open(_file.get(_frame + i), djv::io::READ); }
    catch (Error) {}
  }*/

  const image::Image * image = 0;
  try { image = _load->get(_frame, 0, 0); }
  catch (Error) {}
  if (image)
  {
    _widget->set(image);
    _widget->profile(image->profile);
    _widget->redraw();
  }
  
  ++_frame;
  if (_frame > _time.end) _frame = _time.start;
}

void update_callback(void * in) { reinterpret_cast<App *>(in)->update(); }

}

int main(int argc, char ** argv) { return ::App(argc, argv).exec(); }

