//#define DJV_DBG

#include <djv_base/file.h>
#include <djv_image/io.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/window.h>
#include <djv_ui/image.h>
#include <djv_ui/app.h>
#include <FL/Fl_Menu.H>

namespace
{
using namespace djv;

class App : public ui::App
{
public:

  App(int & argc, char ** argv);
  ~App();
  
private:

  void open(const File &);
  
  DJV_CALLBACK(App, exit_callback, bool) { exit(); }

  image::io::Load * _load;
  ui::window::Widget * _window;
  ui::image::Widget * _widget;
};

App::App(int & argc, char ** argv) :
  ui::App("image_view", argc, argv),
  _window(0)
{
  DJV_DBG_OBJ("App::App");
  
  if (argc != 2)
  {
    print("Usage: image_view (input)");
    image::App::exit(app::EXIT_ERROR);
    return;
  }
  
  const image::Image * image = 0;
  File file(argv[1]);
  try
  {
    _load = image::io::load(file);
    image = _load->get();
  }
  catch (Error in)
  {
    error(str::Format("%1: %2").arg(file).arg(in));
    image::App::exit(app::EXIT_ERROR);
    return;
  }
  
  // Widgets.

  _window = new ui::window::Widget(name());
  _widget = new ui::image::Widget;
  _widget->set(image);
  if (image) _widget->profile(image->profile);
    
  // Layout.

  ui::layout::Vertical * layout = new ui::layout::Vertical(_window);
  layout->spacing(0);
  layout->margin(0);
  layout->add(_widget);
  layout->stretch(_widget);
  
  // Initialize.
  
  if (image) _window->size(image->size());
  _window->show();

  // Callbacks.
  
  _window->close_sig.set(this, exit_callback );

  valid(true);
}

App::~App()
{
  DJV_DBG_OBJ("App::~App");
  
  if (_window) _window->widget_del();
}

}

int main(int argc, char ** argv) { return ::App(argc, argv).exec(); }

